# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from django.urls import reverse

from sahara_dashboard import api
from sahara_dashboard.test import helpers as test


INDEX_URL = reverse('horizon:project:data_processing.jobs:index')
DETAILS_URL = reverse(
    'horizon:project:data_processing.jobs:jb-details', args=['id'])
EDIT_URL = reverse('horizon:project:data_processing.jobs'
                   ':edit-job-binary', args=['id'])
CREATE_URL = reverse(
    'horizon:project:data_processing.jobs:create-job-binary')


class DataProcessingJobBinaryTests(test.TestCase):

    @test.create_mocks({api.sahara: ('job_execution_list',
                                     'plugin_list', 'job_binary_list',
                                     'data_source_list',
                                     'job_list')})
    def test_index(self):
        self.mock_job_binary_list.return_value = \
            self.job_binaries.list()
        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'jobs/index.html')
        self.assertContains(res, 'Job Binaries')
        self.assertContains(res, 'Name')
        self.assertContains(res, 'example.pig')
        self.mock_job_binary_list.assert_called_once_with(
            test.IsHttpRequest())

    @test.create_mocks({api.sahara: ('job_binary_get',)})
    def test_details(self):
        self.mock_job_binary_get.return_value = self.job_binaries.first()
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_job_binary_get, 2,
            mock.call(test.IsHttpRequest(), test.IsA(str)))

    @test.create_mocks({api.sahara: ('job_binary_list',
                                     'job_binary_get',
                                     'job_binary_internal_delete',
                                     'job_binary_delete',)})
    def test_delete(self):
        jb_list = self.mock_job_binary_list.return_value = \
            self.job_binaries.list()
        self.mock_job_binary_get.return_value = self.job_binaries.list()[0]
        self.mock_job_binary_delete.return_value = None
        int_id = jb_list[0].url.split("//")[1]
        self.mock_job_binary_internal_delete.return_value = None
        form_data = {"action": "job_binaries__delete__%s" % jb_list[0].id}
        res = self.client.post(INDEX_URL, form_data)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_job_binary_list.assert_called_once_with(
            test.IsHttpRequest())
        self.mock_job_binary_get.assert_called_once_with(
            test.IsHttpRequest(), test.IsA(str))
        self.mock_job_binary_delete.assert_called_once_with(
            test.IsHttpRequest(), jb_list[0].id)
        self.mock_job_binary_internal_delete.assert_called_once_with(
            test.IsHttpRequest(), int_id)

    @test.create_mocks({api.sahara: ('job_binary_get',
                                     'job_binary_get_file')})
    def test_download(self):
        jb = self.mock_job_binary_get.return_value = \
            self.job_binaries.list()[0]
        self.mock_job_binary_get_file.return_value = \
            "TEST FILE CONTENT"

        context = {'job_binary_id': jb.id}
        url = reverse('horizon:project:data_processing.jobs:download',
                      kwargs={'job_binary_id': jb.id})
        res = self.client.get(url, context)
        self.assertTrue(res.has_header('content-disposition'))

        self.mock_job_binary_get.assert_called_once_with(
            test.IsHttpRequest(), test.IsA(str))
        self.mock_job_binary_get_file.assert_called_once_with(
            test.IsHttpRequest(), jb.id)

    @test.create_mocks({api.sahara: ('job_binary_get',
                                     'job_binary_get_file')})
    def test_download_with_spaces(self):
        jb = self.mock_job_binary_get.return_value = \
            self.job_binaries.list()[1]
        self.mock_job_binary_get_file.return_value = \
            "MORE TEST FILE CONTENT"

        context = {'job_binary_id': jb.id}
        url = reverse('horizon:project:data_processing.jobs:download',
                      kwargs={'job_binary_id': jb.id})
        res = self.client.get(url, context)
        self.assertEqual(
            res.get('Content-Disposition'),
            'attachment; filename="%s"' % jb.name
        )
        self.mock_job_binary_get.assert_called_once_with(
            test.IsHttpRequest(), test.IsA(str))
        self.mock_job_binary_get_file.assert_called_once_with(
            test.IsHttpRequest(), jb.id)

    @test.create_mocks({api.sahara: ('job_binary_get',
                                     'job_binary_update',
                                     'job_binary_internal_list')})
    def test_update(self):
        jb = self.mock_job_binary_get.return_value = \
            self.job_binaries.first()
        self.mock_job_binary_update.return_value = \
            self.job_binaries.first()

        form_data = {
            'job_binary_url': jb.url,
            'job_binary_name': jb.name,
            'job_binary_description': jb.description,
            'job_binary_type': "internal-db",
            'job_binary_internal': "",
            'job_binary_file': "",
            'job_binary_password': "",
            'job_binary_username': "",
            'job_binary_script': "",
            'job_binary_script_name': ""
        }
        res = self.client.post(EDIT_URL, form_data)
        self.assertNoFormErrors(res)
        self.mock_job_binary_get.assert_called_once_with(
            test.IsHttpRequest(), test.IsA(str))
        self.mock_job_binary_update.assert_called_once_with(
            test.IsHttpRequest(), test.IsA(str), test.IsA(dict))

    @test.create_mocks({api.manila: ('share_list', ),
                        api.sahara: ('job_binary_create',
                                     'job_binary_internal_list'),
                        api.sahara.base: ('is_service_enabled', )})
    def test_create_manila(self):
        share = mock.Mock()
        share.return_value = {"id": "tuvwxy56-1234-abcd-abcd-defabcdaedcb",
                              "name": "Test share"}
        shares = [share]
        self.mock_is_service_enabled.return_value = True
        self.mock_share_list.return_value = shares

        form_data = {
            "job_binary_type": "manila",
            "job_binary_manila_share": share.id,
            "job_binary_manila_path": "/testfile.bin",
            "job_binary_name": "testmanila",
            "job_binary_description": "Test manila description"
        }

        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_is_service_enabled, 1,
            mock.call(test.IsHttpRequest(), test.IsA(str)))

    @test.create_mocks({api.sahara: ('job_binary_create',
                                     'job_binary_internal_list')})
    def test_create_s3(self):
        form_data = {
            "job_binary_type": "s3",
            "job_binary_s3_url": "s3://a/b",
            "job_binary_access_key": "acc",
            "job_binary_secret_key": "sec",
            "job_binary_s3_endpoint": "http://pointy.end",
            "job_binary_name": "tests3",
            "job_binary_description": "Test s3 description"
        }
        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins


class JobsPage(mixins.DeleteMixin, basepage.BaseDataProcessingPage):

    TABLE_NAME = "jobs"

    def _get_row_by_template_name(self, name):
        return self.table.get_row('job_name', name)

    def is_job_succeeded(self, name):
        row = self._get_row_by_template_name(name)
        status = row.cells['status'].text
        if status.startswith('Error'):
            raise Exception('Job {} status is {}'.format(name, status))
        return status == "Succeeded"

    def delete_many(self, names=()):
        for name in names:
            row = self._get_row_by_template_name(name)
            if row is not None:
                row.mark()
        self.table.get_delete_form().submit()

    def delete(self, name):
        self.delete_many([name])

    def wait_until_job_succeeded(self, name, timeout=None):
        self._wait_until(lambda x: self.is_job_succeeded(name),
                         timeout=timeout)

    def get_details(self, template_name):
        row = self._get_row_by_template_name(template_name)
        row.cells['name'].find_element_by_tag_name('a').click()
        details = {}
        self.driver.implicitly_wait(0)
        items = self.driver.find_elements_by_css_selector('div.detail dt')
        for item in items:
            key = item.text
            value_elem = item.find_element_by_xpath('./following-sibling::*')
            if value_elem.tag_name != "dd":
                continue
            sub_elements = value_elem.find_elements_by_css_selector(
                'li > span')
            if sub_elements:
                value = {}
                for sub_element in sub_elements:
                    sub_key = sub_element.text.strip(':')
                    value[sub_key] = set()
                    for elem in sub_element.find_elements_by_xpath(
                        './following-sibling::ul/li'
                    ):
                        value[sub_key].add(elem.text)
            else:
                value = value_elem.text
            details[key] = value
        self.driver.implicitly_wait(30)
        return details



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import tabs

from sahara_dashboard.content.data_processing.clusters.cluster_templates \
    import tabs as cluster_templates_tabs
from sahara_dashboard.content.data_processing.clusters.clusters \
    import tabs as clusters_tabs
from sahara_dashboard.content.data_processing.clusters.image_registry \
    import tabs as image_registry_tabs
from sahara_dashboard.content.data_processing.clusters.nodegroup_templates \
    import tabs as node_group_templates_tabs
from sahara_dashboard.content.data_processing.tabs \
    import PaginationFriendlyTabGroup


class ClusterTabs(PaginationFriendlyTabGroup):
    slug = "cluster_tabs"
    tabs = (clusters_tabs.ClustersTab,
            cluster_templates_tabs.ClusterTemplatesTab,
            node_group_templates_tabs.NodeGroupTemplatesTab,
            image_registry_tabs.ImageRegistryTab,)
    sticky = True


class IndexView(tabs.TabbedTableView):
    tab_group_class = ClusterTabs
    template_name = "clusters/index.html"
    page_title = _("Clusters")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from django.urls import re_path

import sahara_dashboard.content.data_processing.clusters.views as views
import sahara_dashboard.content.data_processing.clusters.image_registry. \
    views as image_views
import sahara_dashboard.content.data_processing.clusters.nodegroup_templates. \
    views as ngt_views
import sahara_dashboard.content.data_processing.clusters.cluster_templates. \
    views as ct_views
import sahara_dashboard.content.data_processing.clusters.clusters. \
    views as cluster_views
import sahara_dashboard.content.data_processing.clusters.wizard. \
    views as cluster_guide_views


urlpatterns = [re_path(r'^$', views.IndexView.as_view(), name='index'),
               re_path(r'^\?tab=cluster_tabs__image_registry_tab$',
                       views.IndexView.as_view(), name='image-registry-tab'),
               re_path(r'^\?tab=cluster_tabs__node_group_templates_tab$',
                       views.IndexView.as_view(),
                       name='nodegroup-templates-tab'),
               re_path(r'^\?tab=cluster_tabs__clusters_templates_tab$',
                       views.IndexView.as_view(),
                       name='cluster-templates-tab'),
               re_path(r'^\?tab=cluster_tabs__clusters_tab$',
                       views.IndexView.as_view(), name='clusters-tab'),
               re_path(r'^create-cluster-template$',
                       ct_views.CreateClusterTemplateView.as_view(),
                       name='create-cluster-template'),
               re_path(r'^create-nodegroup-template$',
                       ngt_views.CreateNodegroupTemplateView.as_view(),
                       name='create-nodegroup-template'),
               re_path(r'^configure-cluster-template$',
                       ct_views.ConfigureClusterTemplateView.as_view(),
                       name='configure-cluster-template'),
               re_path(r'^import-cluster-template-file$',
                       ct_views.ImportClusterTemplateFileView.as_view(),
                       name='import-cluster-template-file'),
               re_path(r'^import-cluster-template-name$',
                       ct_views.ImportClusterTemplateNameView.as_view(),
                       name='import-cluster-template-name'),
               re_path(r'^import-cluster-template-nodegroups$',
                       ct_views.ImportClusterTemplateNodegroupsView.as_view(),
                       name='import-cluster-template-nodegroups'),
               re_path(r'^configure-nodegroup-template$',
                       ngt_views.ConfigureNodegroupTemplateView.as_view(),
                       name='configure-nodegroup-template'),
               re_path(r'^import-nodegroup-template-file$',
                       ngt_views.ImportNodegroupTemplateFileView.as_view(),
                       name='import-nodegroup-template-file'),
               re_path(r'^import-nodegroup-template-details$',
                       ngt_views.ImportNodegroupTemplateDetailsView.as_view(),
                       name='import-nodegroup-template-details'),
               re_path(r'^cluster-template/(?P<template_id>[^/]+)$',
                       ct_views.ClusterTemplateDetailsView.as_view(),
                       name='ct-details'),
               re_path(r'^node-group-template/(?P<template_id>[^/]+)$',
                       ngt_views.NodegroupTemplateDetailsView.as_view(),
                       name='details'),
               re_path(r'^cluster-template/(?P<template_id>[^/]+)/copy$',
                       ct_views.CopyClusterTemplateView.as_view(),
                       name='ct-copy'),
               re_path(r'^cluster-template/(?P<template_id>[^/]+)/edit$',
                       ct_views.EditClusterTemplateView.as_view(),
                       name='ct-edit'),
               re_path(r'^node-group-template/(?P<template_id>[^/]+)/copy$',
                       ngt_views.CopyNodegroupTemplateView.as_view(),
                       name='copy'),
               re_path(r'^node-group-template/(?P<template_id>[^/]+)/edit$',
                       ngt_views.EditNodegroupTemplateView.as_view(),
                       name='edit'),
               re_path(r'^create-cluster$',
                       cluster_views.CreateClusterView.as_view(),
                       name='create-cluster'),
               re_path(r'^configure-cluster$',
                       cluster_views.ConfigureClusterView.as_view(),
                       name='configure-cluster'),
               re_path(r'^cluster/(?P<cluster_id>[^/]+)$',
                       cluster_views.ClusterDetailsView.as_view(),
                       name='cluster-details'),
               re_path(r'^cluster/(?P<cluster_id>[^/]+)/events$',
                       cluster_views.ClusterEventsView.as_view(),
                       name='events'),
               re_path(r'^cluster/(?P<cluster_id>[^/]+)/scale$',
                       cluster_views.ScaleClusterView.as_view(),
                       name='scale'),
               re_path(r'^cluster/(?P<cluster_id>[^/]+)/verifications$',
                       cluster_views.ClusterHealthChecksView.as_view(),
                       name='verifications'),
               re_path(r'^cluster/(?P<cluster_id>[^/]+)/update_shares$',
                       cluster_views.UpdateClusterSharesView.as_view(),
                       name='update-shares'),
               re_path(r'^edit_tags/(?P<image_id>[^/]+)/$',
                       image_views.EditTagsView.as_view(), name='edit_tags'),
               re_path(r'^register/$',
                       image_views.RegisterImageView.as_view(),
                       name='register'),
               re_path(r'^cluster_guide$',
                       cluster_guide_views.ClusterGuideView.as_view(),
                       name='cluster_guide'),
               re_path(r'^cluster_guide/(?P<reset_cluster_guide>[^/]+)/$',
                       cluster_guide_views.ResetClusterGuideView.as_view(),
                       name='reset_cluster_guide'),
               re_path(r'^image_register/$',
                       cluster_guide_views.ImageRegisterView.as_view(),
                       name='image_register'),
               re_path(r'^plugin_select$',
                       cluster_guide_views.PluginSelectView.as_view(),
                       name='plugin_select'),
               re_path(r'^ngt_select$',
                       cluster_guide_views.NodeGroupSelectView.as_view(),
                       name='ngt_select'), ]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.tables as _tables
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.tabs as _tabs
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.workflows.copy as copy_flow
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.workflows.create as create_flow
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.workflows.edit as edit_flow
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.forms.import_forms as import_forms


class NodegroupTemplateDetailsView(tabs.TabView):
    tab_group_class = _tabs.NodegroupTemplateDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ template.name|default:template.id }}"

    @memoized.memoized_method
    def get_object(self):
        ngt_id = self.kwargs["template_id"]
        try:
            return saharaclient.nodegroup_template_get(self.request, ngt_id)
        except Exception:
            msg = _('Unable to retrieve details for '
                    'node group template "%s".') % ngt_id
            redirect = self.get_redirect_url()
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(
            NodegroupTemplateDetailsView, self).get_context_data(**kwargs)
        node_group_template = self.get_object()
        context['template'] = node_group_template
        context['url'] = self.get_redirect_url()
        context['actions'] = self._get_actions(node_group_template)
        return context

    def _get_actions(self, node_group_template):
        table = _tables.NodegroupTemplatesTable(self.request)
        return table.render_row_actions(node_group_template)

    @staticmethod
    def get_redirect_url():
        return reverse("horizon:project:data_processing."
                       "clusters:index")


class CreateNodegroupTemplateView(workflows.WorkflowView):
    workflow_class = create_flow.CreateNodegroupTemplate
    success_url = (
        "horizon:project:data_processing.clusters:"
        "create-nodegroup-template")
    classes = ("ajax-modal",)
    template_name = "nodegroup_templates/create.html"
    page_title = _("Create Node Group Template")


class ConfigureNodegroupTemplateView(workflows.WorkflowView):
    workflow_class = create_flow.ConfigureNodegroupTemplate
    success_url = ("horizon:project:"
                   "data_processing.clusters:index")
    template_name = "nodegroup_templates/configure.html"
    page_title = _("Create Node Group Template")

    def get_initial(self):
        initial = super(ConfigureNodegroupTemplateView, self).get_initial()
        initial.update(self.kwargs)
        return initial


class CopyNodegroupTemplateView(workflows.WorkflowView):
    workflow_class = copy_flow.CopyNodegroupTemplate
    success_url = ("horizon:project:"
                   "data_processing.clusters:index")
    template_name = "nodegroup_templates/configure.html"

    def get_context_data(self, **kwargs):
        context = super(
            CopyNodegroupTemplateView, self).get_context_data(**kwargs)

        context["template_id"] = kwargs["template_id"]
        return context

    def get_object(self, *args, **kwargs):
        if not hasattr(self, "_object"):
            template_id = self.kwargs['template_id']
            try:
                template = saharaclient.nodegroup_template_get(self.request,
                                                               template_id)
            except Exception:
                template = None
                exceptions.handle(self.request,
                                  _("Unable to fetch template object."))
            self._object = template
        return self._object

    def get_initial(self):
        initial = super(CopyNodegroupTemplateView, self).get_initial()
        initial['template_id'] = self.kwargs['template_id']
        return initial


class EditNodegroupTemplateView(CopyNodegroupTemplateView):
    workflow_class = edit_flow.EditNodegroupTemplate
    success_url = ("horizon:project:"
                   "data_processing.clusters:index")
    template_name = "nodegroup_templates/configure.html"


class ImportNodegroupTemplateFileView(forms.ModalFormView):
    template_name = "nodegroup_templates/import.html"
    form_class = import_forms.ImportNodegroupTemplateFileForm
    submit_label = _("Next")
    submit_url = reverse_lazy("horizon:project:data_processing."
                              "clusters:import-nodegroup-template-file")
    success_url = reverse_lazy("horizon:project:data_processing."
                               "clusters:import-nodegroup-template-details")
    page_title = _("Import Node Group Template")

    def get_form_kwargs(self):
        kwargs = super(
            ImportNodegroupTemplateFileView, self).get_form_kwargs()
        kwargs['next_view'] = ImportNodegroupTemplateDetailsView
        return kwargs


class ImportNodegroupTemplateDetailsView(forms.ModalFormView):
    template_name = "nodegroup_templates/import.html"
    form_class = import_forms.ImportNodegroupTemplateDetailsForm
    submit_label = _("Import")
    submit_url = reverse_lazy("horizon:project:data_processing."
                              "clusters:import-nodegroup-template-details")
    success_url = reverse_lazy("horizon:project:data_processing."
                               "clusters:index")
    page_title = _("Import Node Group Template")

    def get_form_kwargs(self):
        kwargs = super(
            ImportNodegroupTemplateDetailsView, self).get_form_kwargs()
        if 'template_upload' in self.kwargs:
            kwargs['template_upload'] = self.kwargs['template_upload']
        return kwargs



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs
from sahara_dashboard.content.data_processing \
    .jobs.jobs import tables as jobs_tables

LOG = logging.getLogger(__name__)


class JobsTab(sahara_tabs.SaharaTableTab):
    table_classes = (jobs_tables.JobsTable, )
    name = _("Jobs")
    slug = "jobs_tab"
    template_name = "horizon/common/_detail_table.html"
    SEARCH_MAPPING = {"cluster": "cluster.name",
                      "job": "job.name"}

    def get_jobs_data(self):
        try:
            table = self._tables['jobs']
            search_opts = {}
            filter = self.get_server_filter_info(table.request, table)
            if filter['value'] and filter['field']:
                if filter['field'] in self.SEARCH_MAPPING:
                    # Handle special cases for cluster and job
                    # since they are in different database tables.
                    search_opts = {
                        self.SEARCH_MAPPING[filter['field']]: filter['value']}
                else:
                    search_opts = {filter['field']: filter['value']}
            jobs = saharaclient.job_execution_list(self.request, search_opts)
        except Exception:
            jobs = []
            exceptions.handle(self.request,
                              _("Unable to fetch job list"))
        return jobs


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "job_execution_tab"
    template_name = "jobs/_details.html"

    def get_context_data(self, request):
        jex_id = self.tab_group.kwargs['job_execution_id']
        try:
            job_execution = saharaclient.job_execution_get(request, jex_id)
        except Exception as e:
            job_execution = {}
            LOG.error("Unable to fetch job details: %s" % str(e))
            return {"job_execution": job_execution}
        object_names = self.get_object_names(job_execution,
                                             request)
        try:
            object_names['input_url'] = job_execution.data_source_urls.get(
                job_execution.input_id)
        except Exception as e:
            LOG.error("Unable to fetch input url: %s", e)
            object_names["input_url"] = "None"

        try:
            object_names['output_url'] = job_execution.data_source_urls.get(
                job_execution.output_id)
        except Exception as e:
            LOG.error("Unable to fetch output url: %s", e)
            object_names["output_url"] = "None"

        if saharaclient.VERSIONS.active == '2':
            job_execution_engine_job_attr = "engine_job_id"
            job_execution_engine_job_attr_pretty = _("Engine Job ID")
        else:
            job_execution_engine_job_attr = "oozie_job_id"
            job_execution_engine_job_attr_pretty = _("Oozie Job ID")

        job_execution_engine_job_id = getattr(job_execution,
                                              job_execution_engine_job_attr)

        return {"job_execution": job_execution,
                "object_names": object_names,
                "job_execution_engine_job_id": job_execution_engine_job_id,
                "job_execution_engine_job_attr_pretty":
                    job_execution_engine_job_attr_pretty}

    def get_object_names(self, job_ex, request):
        object_names = {}
        try:
            job_template_id = job_ex.job_template_id  # typical APIv2
        except AttributeError:
            job_template_id = job_ex.job_id  # APIv1.1, older APIv2
        obj_names_map = {'input_name': {'obj': 'data_source_get',
                                        'obj_id': job_ex.input_id},
                         'output_name': {'obj': 'data_source_get',
                                         'obj_id': job_ex.output_id},
                         'cluster_name': {'obj': 'cluster_get',
                                          'obj_id': job_ex.cluster_id},
                         'job_name': {'obj': 'job_get',
                                      'obj_id': job_template_id}}
        for item in obj_names_map:
            object_names[item] = (
                self.get_object_name(obj_names_map[item]['obj_id'],
                                     obj_names_map[item]['obj'],
                                     request))

        return object_names

    def get_object_name(self, obj_id, sahara_obj, request):
        object_name = None
        try:
            s_func = getattr(saharaclient, sahara_obj)
            obj = s_func(request, obj_id)
            object_name = obj.name
        except Exception as e:
            LOG.warning("Unable to get name for %s with object_id %s (%s)"
                        % (sahara_obj, obj_id, str(e)))
        return object_name


class JobExecutionDetailsTabs(tabs.TabGroup):
    slug = "job_details"
    tabs = (GeneralTab,)
    sticky = True



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins


class CreateMixin(object):
    CREATE_FIELD_MAPPING = (
        "data_source_name",
        "data_source_type",
        "data_source_url",
        "data_source_credential_user",
        "data_source_credential_pass",
        "data_source_description"
    )

    @tables.bind_table_action('create data source')
    def get_create_form(self, button):
        button.click()
        return forms.FormRegion(self.driver, self.conf,
                                field_mappings=self.CREATE_FIELD_MAPPING)

    @tables.bind_row_action('edit data source')
    def get_update_form(self, button, row):
        button.click()
        return forms.FormRegion(self.driver, self.conf,
                                field_mappings=self.CREATE_FIELD_MAPPING)


class DatasourcesPage(mixins.DeleteMixin, basepage.BaseDataProcessingPage):

    TABLE_NAME = "data_sources"

    @classmethod
    def get_table_mixins(cls):
        return super(DatasourcesPage, cls).get_table_mixins() + (CreateMixin,)

    def create(self, name, source_type, url):
        form = self.table.get_create_form()
        form.data_source_name.text = name
        form.data_source_type.text = source_type
        form.data_source_url.text = url
        form.submit()

    def update(self, name, **kwargs):
        row = self._get_row_with_name(name)
        form = self.table.get_update_form(row)
        for key in CreateMixin.CREATE_FIELD_MAPPING:
            if key in kwargs:
                getattr(form, key).text = kwargs[key]
        form.submit()

    def get_details(self, name):
        details = {}
        self.table.src_elem.find_element_by_link_text(name).click()
        items = self.driver.find_elements_by_css_selector('div.detail dt')
        for item in items:
            key = item.text
            value_elem = item.find_element_by_xpath('./following-sibling::*')
            if value_elem.tag_name != "dd":
                continue
            value = value_elem.text
            details[key] = value
        return details



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from openstack_dashboard.api import neutron
from openstack_dashboard.api import nova

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs
from sahara_dashboard.content. \
    data_processing.utils import workflow_helpers as helpers
from sahara_dashboard.content.data_processing.clusters.nodegroup_templates \
    import tables as node_group_template_tables

LOG = logging.getLogger(__name__)


class NodeGroupTemplatesTab(sahara_tabs.SaharaTableTab):
    table_classes = (node_group_template_tables.NodegroupTemplatesTable, )
    name = _("Node Group Templates")
    slug = "node_group_templates_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_nodegroup_templates_data(self):
        try:
            table = self._tables['nodegroup_templates']
            search_opts = {}
            filter = self.get_server_filter_info(table.request, table)
            if filter['value'] and filter['field']:
                search_opts = {filter['field']: filter['value']}
            node_group_templates = saharaclient.nodegroup_template_list(
                self.request, search_opts)
        except Exception:
            node_group_templates = []
            exceptions.handle(self.request,
                              _("Unable to fetch node group template list"))
        return node_group_templates


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "nodegroup_template_details_tab"
    template_name = "nodegroup_templates/_details.html"

    def get_context_data(self, request):
        template_id = self.tab_group.kwargs['template_id']
        try:
            template = saharaclient.nodegroup_template_get(
                request, template_id)
        except Exception as e:
            template = {}
            LOG.error(
                "Unable to fetch node group template details: %s" % str(e))
            return {"template": template}

        try:
            flavor = nova.flavor_get(request, template.flavor_id)
        except Exception:
            flavor = {}
            exceptions.handle(request,
                              _("Unable to fetch flavor for template."))

        floating_ip_pool_name = None
        if template.floating_ip_pool:
            try:
                floating_ip_pool_name = self._get_floating_ip_pool_name(
                    request, template.floating_ip_pool)
            except Exception:
                exceptions.handle(request,
                                  _("Unable to fetch floating ip pools."))

        base_image_name = None
        if template.image_id:
            try:
                base_image_name = saharaclient.image_get(
                    request, template.image_id).name
            except Exception:
                exceptions.handle(request,
                                  _("Unable to fetch Base Image with id: %s.")
                                  % template.image_id)

        security_groups = helpers.get_security_groups(
            request, template.security_groups)

        if getattr(template, 'boot_from_volume', None) is None:
            show_bfv = False
        else:
            show_bfv = True

        return {"template": template, "flavor": flavor,
                "floating_ip_pool_name": floating_ip_pool_name,
                "base_image_name": base_image_name,
                "security_groups": security_groups,
                "show_bfv": show_bfv}

    def _get_floating_ip_pool_name(self, request, pool_id):
        pools = [pool for pool in neutron.floating_ip_pools_list(
            request) if pool.id == pool_id]

        return pools[0].name if pools else pool_id


class ConfigsTab(tabs.Tab):
    name = _("Service Configurations")
    slug = "nodegroup_template_service_configs_tab"
    template_name = "nodegroup_templates/_service_confs.html"

    def get_context_data(self, request):
        template_id = self.tab_group.kwargs['template_id']
        try:
            template = saharaclient.nodegroup_template_get(
                request, template_id)
        except Exception as e:
            template = {}
            LOG.error(
                "Unable to fetch node group template details: %s" % str(e))
        return {"template": template}


class NodegroupTemplateDetailsTabs(tabs.TabGroup):
    slug = "nodegroup_template_details"
    tabs = (GeneralTab, ConfigsTab, )
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs
from sahara_dashboard.content.data_processing.jobs \
    .job_templates import tables as job_templates_tables

LOG = logging.getLogger(__name__)


class JobTemplatesTab(sahara_tabs.SaharaTableTab):
    table_classes = (job_templates_tables.JobTemplatesTable, )
    name = _("Job Templates")
    slug = "job_templates_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_job_templates_data(self):
        try:
            table = self._tables['job_templates']
            search_opts = {}
            filter = self.get_server_filter_info(table.request, table)
            if filter['value'] and filter['field']:
                search_opts = {filter['field']: filter['value']}
            job_templates = saharaclient.job_list(self.request, search_opts)
        except Exception:
            job_templates = []
            exceptions.handle(self.request,
                              _("Unable to fetch job template list"))
        return job_templates


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "job_details_tab"
    template_name = "job_templates/_details.html"

    def get_context_data(self, request):
        job_id = self.tab_group.kwargs['job_id']
        try:
            job = saharaclient.job_get(request, job_id)
        except Exception as e:
            job = {}
            LOG.error("Unable to fetch job template details: %s" % str(e))
        return {"job": job}


class JobDetailsTabs(tabs.TabGroup):
    slug = "job_details"
    tabs = (GeneralTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _
from sahara_dashboard.api import sahara as saharaclient

from horizon import exceptions
from horizon import tables
from horizon import tabs
from openstack_dashboard.api import glance
from openstack_dashboard.api import neutron
from openstack_dashboard.api import nova

from sahara_dashboard.content.data_processing.clusters.clusters \
    import tables as cluster_tables
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs
from sahara_dashboard.content.data_processing.utils \
    import workflow_helpers as helpers

LOG = logging.getLogger(__name__)


class ClustersTab(sahara_tabs.SaharaTableTab):
    table_classes = (cluster_tables.ClustersTable, )
    name = _("Clusters")
    slug = "clusters_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_clusters_data(self):
        try:
            table = self._tables['clusters']
            search_opts = {}
            filter = self.get_server_filter_info(table.request, table)
            if filter['value'] and filter['field']:
                search_opts = {filter['field']: filter['value']}
            clusters = saharaclient.cluster_list(self.request, search_opts)
        except Exception:
            clusters = []
            exceptions.handle(self.request,
                              _("Unable to fetch cluster list"))
        return clusters


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "cluster_details_tab"
    template_name = "clusters/_details.html"

    def get_context_data(self, request):
        cluster_id = self.tab_group.kwargs['cluster_id']
        cluster_info = {}
        try:
            sahara = saharaclient.client(request)
            cluster = sahara.clusters.get(cluster_id)
            for info_key, info_val in cluster.info.items():
                for key, val in info_val.items():
                    if str(val).startswith(('http://', 'https://')):
                        cluster.info[info_key][key] = build_link(val)

            try:
                base_image = glance.image_get(request,
                                              cluster.default_image_id)
            except Exception:
                exceptions.handle(
                    request, _("Unable to fetch base image details"))
                base_image = {}

            if getattr(cluster, 'cluster_template_id', None):
                cluster_template = saharaclient.safe_call(
                    sahara.cluster_templates.get,
                    cluster.cluster_template_id)
            else:
                cluster_template = None
            try:
                if getattr(cluster, 'neutron_management_network', None):
                    net_id = cluster.neutron_management_network
                    network = neutron.network_get(request, net_id)
                    net_name = network.name_or_id
                else:
                    net_name = None
            except Exception:
                exceptions.handle(
                    request, _("Unable to fetch network details"))
                net_name = None

            cluster_info.update({"cluster": cluster,
                                 "base_image": base_image,
                                 "cluster_template": cluster_template,
                                 "network": net_name})

            if saharaclient.VERSIONS.active == '2':
                cluster_info["cluster"].hadoop_version = (
                    cluster_info["cluster"].plugin_version
                )
        except Exception as e:
            LOG.error("Unable to fetch cluster details: %s" % str(e))

        return cluster_info


class ClusterConfigsDetails(tabs.Tab):
    name = _("Configuration Details")
    slug = "cluster_configs_details_tab"
    template_name = (
        "clusters/_cluster_configs_details.html")

    def get_context_data(self, request):
        cluster_id = self.tab_group.kwargs['cluster_id']
        cluster = {}
        try:
            sahara = saharaclient.client(request)
            cluster = sahara.clusters.get(cluster_id)

        except Exception as e:
            LOG.error("Unable to fetch cluster details: %s" % str(e))

        return {'cluster': cluster}


def build_link(url):
    return "<a href='" + url + "' target=\"_blank\">" + url + "</a>"


class NodeGroupsTab(tabs.Tab):
    name = _("Node Groups")
    slug = "cluster_nodegroups_tab"
    template_name = "clusters/_nodegroups_details.html"

    def get_context_data(self, request):
        cluster_id = self.tab_group.kwargs['cluster_id']
        try:
            sahara = saharaclient.client(request)
            cluster = sahara.clusters.get(cluster_id)
            for ng in cluster.node_groups:
                if ng["flavor_id"]:
                    ng["flavor_name"] = (
                        nova.flavor_get(request, ng["flavor_id"]).name)
                if ng["floating_ip_pool"]:
                    ng["floating_ip_pool_name"] = (
                        self._get_floating_ip_pool_name(
                            request, ng["floating_ip_pool"]))

                if ng.get("node_group_template_id", None):
                    ng["node_group_template"] = saharaclient.safe_call(
                        sahara.node_group_templates.get,
                        ng["node_group_template_id"])

                ng["security_groups_full"] = helpers.get_security_groups(
                    request, ng["security_groups"])
        except Exception:
            cluster = {}
            exceptions.handle(request,
                              _("Unable to get node group details."))

        return {"cluster": cluster}

    def _get_floating_ip_pool_name(self, request, pool_id):
        pools = [pool for pool in neutron.floating_ip_pools_list(
            request) if pool.id == pool_id]

        return pools[0].name if pools else pool_id


class Instance(object):
    def __init__(self, name=None, id=None, internal_ip=None,
                 management_ip=None):
        self.name = name
        self.id = id
        self.internal_ip = internal_ip
        self.management_ip = management_ip


class InstancesTable(tables.DataTable):
    name = tables.Column("name",
                         link="horizon:project:instances:detail",
                         verbose_name=_("Name"))

    internal_ip = tables.Column("internal_ip",
                                verbose_name=_("Internal IP"))

    management_ip = tables.Column("management_ip",
                                  verbose_name=_("Management IP"))

    class Meta(object):
        name = "cluster_instances"
        verbose_name = _("Cluster Instances")


class InstancesTab(tabs.TableTab):
    name = _("Instances")
    slug = "cluster_instances_tab"
    template_name = "clusters/_instances_details.html"
    table_classes = (InstancesTable, )

    def get_cluster_instances_data(self):
        cluster_id = self.tab_group.kwargs['cluster_id']

        try:
            sahara = saharaclient.client(self.request)
            cluster = sahara.clusters.get(cluster_id)

            instances = []
            for ng in cluster.node_groups:
                for instance in ng["instances"]:
                    instances.append(Instance(
                        name=instance["instance_name"],
                        id=instance["instance_id"],
                        internal_ip=instance.get("internal_ip",
                                                 "Not assigned"),
                        management_ip=instance.get("management_ip",
                                                   "Not assigned")))
        except Exception:
            instances = []
            exceptions.handle(self.request,
                              _("Unable to fetch instance details."))
        return instances


class EventLogTab(tabs.Tab):
    name = _("Cluster Events")
    slug = "cluster_event_log"
    template_name = "clusters/_event_log.html"

    def get_context_data(self, request, **kwargs):
        cluster_id = self.tab_group.kwargs['cluster_id']
        kwargs["cluster_id"] = cluster_id
        kwargs['data_update_url'] = request.get_full_path()

        return kwargs


class HealthChecksTab(tabs.Tab):
    name = _("Cluster health checks")
    slug = 'cluster_health_checks'
    template_name = "clusters/_health_checks_table.html"

    def get_context_data(self, request, **kwargs):
        cluster_id = self.tab_group.kwargs['cluster_id']
        kwargs['cluster_id'] = cluster_id
        kwargs['data_update_url'] = request.get_full_path()
        return kwargs


class ClusterDetailsTabs(tabs.TabGroup):
    slug = "cluster_details"
    if saharaclient.SAHARA_VERIFICATION_DISABLED:
        tabs = (GeneralTab, ClusterConfigsDetails, NodeGroupsTab, InstancesTab,
                EventLogTab)
    else:
        tabs = (GeneralTab, ClusterConfigsDetails, NodeGroupsTab, InstancesTab,
                EventLogTab, HealthChecksTab)

    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing.jobs. \
    data_sources.tables as ds_tables
import sahara_dashboard.content.data_processing.jobs. \
    data_sources.tabs as _tabs
import sahara_dashboard.content.data_processing.jobs. \
    data_sources.workflows.create as create_flow
import sahara_dashboard.content.data_processing.jobs. \
    data_sources.workflows.edit as edit_flow


class CreateDataSourceView(workflows.WorkflowView):
    workflow_class = create_flow.CreateDataSource
    success_url = \
        "horizon:project:data_processing.jobs:create-data-source"
    classes = ("ajax-modal",)
    template_name = "data_sources/create.html"
    page_title = _("Create Data Source")


class EditDataSourceView(CreateDataSourceView):
    workflow_class = edit_flow.EditDataSource
    page_title = _("Edit Data Source")

    def get_context_data(self, **kwargs):
        context = super(EditDataSourceView, self) \
            .get_context_data(**kwargs)

        context["data_source_id"] = kwargs["data_source_id"]
        return context

    def get_initial(self):
        initial = super(EditDataSourceView, self).get_initial()
        initial['data_source_id'] = self.kwargs['data_source_id']
        return initial


class DataSourceDetailsView(tabs.TabView):
    tab_group_class = _tabs.DataSourceDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ data_source.name|default:data_source.id }}"

    @memoized.memoized_method
    def get_object(self):
        ds_id = self.kwargs["data_source_id"]
        try:
            return saharaclient.data_source_get(self.request, ds_id)
        except Exception:
            msg = _('Unable to retrieve details for data source "%s".') % ds_id
            redirect = self.get_redirect_url()
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(DataSourceDetailsView, self).get_context_data(**kwargs)
        data_source = self.get_object()
        context['data_source'] = data_source
        context['url'] = self.get_redirect_url()
        context['actions'] = self._get_actions(data_source)
        return context

    def _get_actions(self, data_source):
        table = ds_tables.DataSourcesTable(self.request)
        return table.render_row_actions(data_source)

    @staticmethod
    def get_redirect_url():
        return reverse("horizon:project:data_processing.jobs:index")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _
from saharaclient.api import base as api_base

from horizon import exceptions
from horizon import forms
from horizon import workflows

from sahara_dashboard.api import sahara as saharaclient


class UpdateLabelsAction(workflows.Action):
    def __init__(self, request, *args, **kwargs):
        super(UpdateLabelsAction, self).__init__(request, *args, **kwargs)
        plugin_name = [
            x['plugin_name'] for x in args if 'plugin_name' in x][0]
        plugin = saharaclient.plugin_get(request, plugin_name)
        self._serialize_labels(
            'plugin_', _("Plugin label"), plugin.plugin_labels)
        vers_labels = plugin.version_labels
        for version in vers_labels.keys():
            field_label = _("Plugin version %(version)s label") % {
                'version': version}
            self._serialize_labels(
                'version_%s_' % version, field_label, vers_labels[version])
        self.fields["plugin_name"] = forms.CharField(
            widget=forms.HiddenInput(),
            initial=plugin_name)

    def _serialize_labels(self, prefix, prefix_trans, labels):
        for name, label in labels.items():
            if not label['mutable']:
                continue
            res_name_translated = "%s: %s" % (prefix_trans, name)
            res_name = "label_%s%s" % (prefix, name)
            self.fields[res_name] = forms.BooleanField(
                label=res_name_translated,
                help_text=label['description'],
                widget=forms.CheckboxInput(),
                initial=label['status'],
                required=False,
            )

    class Meta(object):
        name = _("Plugin")
        help_text = _("Update the plugin labels")


class UpdatePluginStep(workflows.Step):
    action_class = UpdateLabelsAction
    depends_on = ('plugin_name', )

    def contribute(self, data, context):
        for name, item in data.items():
            context[name] = item
        return context


class UpdatePlugin(workflows.Workflow):
    slug = "update_plugin"
    name = _("Update Plugin")
    success_message = _("Updated")
    failure_message = _("Could not update plugin")
    success_url = "horizon:project:data_processing.data_plugins:index"
    default_steps = (UpdatePluginStep,)

    def __init__(self, request, context_seed, entry_point, *args, **kwargs):
        super(UpdatePlugin, self).__init__(
            request, context_seed, entry_point, *args, **kwargs)

    def _get_update_values(self, context):
        values = {'plugin_labels': {}, 'version_labels': {}}
        for item, item_value in context.items():
            if not item.startswith('label_'):
                continue
            name = item.split('_')[1:]
            if name[0] == 'plugin':
                values['plugin_labels'][name[1]] = {'status': item_value}
            else:
                if name[1] not in values['version_labels']:
                    values['version_labels'][name[1]] = {}
                values['version_labels'][
                    name[1]][name[2]] = {'status': item_value}
        return values

    def handle(self, request, context):
        try:
            update_values = self._get_update_values(context)
            saharaclient.plugin_update(
                request, context['plugin_name'], update_values)
            return True
        except api_base.APIException as e:
            self.error_description = str(e)
            return False
        except Exception:
            exceptions.handle(request,
                              _("Plugin update failed."))
            return False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.jobs.job_binaries \
    import tables as job_binary_tables
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs

LOG = logging.getLogger(__name__)


class JobBinariesTab(sahara_tabs.SaharaTableTab):
    table_classes = (job_binary_tables.JobBinariesTable, )
    name = _("Job Binaries")
    slug = "job_binaries_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_job_binaries_data(self):
        try:
            job_binaries = saharaclient.job_binary_list(self.request)
        except Exception:
            job_binaries = []
            exceptions.handle(self.request,
                              _("Unable to fetch job binary list"))
        return job_binaries


class JobBinaryDetailsTab(tabs.Tab):
    name = _("General Info")
    slug = "job_binaries_details_tab"
    template_name = "job_binaries/_details.html"

    def get_context_data(self, request):
        job_binary_id = self.tab_group.kwargs['job_binary_id']
        try:
            job_binary = saharaclient.job_binary_get(request, job_binary_id)
        except Exception as e:
            job_binary = {}
            LOG.error("Unable to fetch job binary details: %s" % str(e))
        return {"job_binary": job_binary}


class JobBinaryDetailsTabs(tabs.TabGroup):
    slug = "job_binary_details"
    tabs = (JobBinaryDetailsTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon.utils import memoized

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.jobs.jobs \
    import tables as je_tables
import sahara_dashboard.content.data_processing \
    .jobs.jobs.tabs as _tabs


class JobDetailsView(tabs.TabView):
    tab_group_class = _tabs.JobExecutionDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ job_execution.name|default:job_execution.id }}"

    @memoized.memoized_method
    def get_object(self):
        jex_id = self.kwargs["job_execution_id"]
        try:
            return saharaclient.job_execution_get(self.request, jex_id)
        except Exception:
            msg = _('Unable to retrieve details for job "%s".') % jex_id
            redirect = self.get_redirect_url()
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(JobDetailsView, self)\
            .get_context_data(**kwargs)
        job_execution = self.get_object()
        context['job_execution'] = job_execution
        context['url'] = self.get_redirect_url()
        context['actions'] = self._get_actions(job_execution)
        return context

    def _get_actions(self, job_execution):
        table = je_tables.JobsTable(self.request)
        return table.render_row_actions(job_execution)

    @staticmethod
    def get_redirect_url():
        return reverse("horizon:project:data_processing.jobs:jobs-tab")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools

from django.utils import encoding
from django.utils import html
from django.utils import safestring
from django.utils.translation import gettext_lazy as _
from oslo_log import log as logging
from oslo_utils import uuidutils
from saharaclient.api import base as api_base

from horizon import exceptions
from horizon import forms
from horizon import workflows
from openstack_dashboard.api import cinder
from openstack_dashboard.api import neutron
from openstack_dashboard.api import nova
from openstack_dashboard.dashboards.project.instances \
    import utils as nova_utils

from sahara_dashboard.api import manila as manilaclient
from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
from sahara_dashboard.content.data_processing.utils \
    import helpers
from sahara_dashboard.content.data_processing.utils \
    import workflow_helpers

LOG = logging.getLogger(__name__)

BASE_IMAGE_URL = "horizon:project:data_processing.clusters:register"


def is_cinder_enabled(request):
    for service in ['volumev3', 'volumev2', 'volume']:
        if saharaclient.base.is_service_enabled(request, service):
            return True
    return False


def storage_choices(request):
    choices = [("ephemeral_drive", _("Ephemeral Drive")), ]
    if is_cinder_enabled(request):
        choices.append(("cinder_volume", _("Cinder Volume")))
    else:
        LOG.warning(_("Cinder service is unavailable now"))
    return choices


def volume_availability_zone_list(request):
    """Utility method to retrieve a list of volume availability zones."""
    try:
        if cinder.extension_supported(request, 'AvailabilityZones'):
            return cinder.availability_zone_list(request)
        return []
    except Exception:
        exceptions.handle(request,
                          _('Unable to retrieve volumes availability zones.'))
        return []


def instance_availability_zone_list(request):
    """Utility method to retrieve a list of instance availability zones."""
    try:
        return nova.availability_zone_list(request)
    except Exception:
        exceptions.handle(request,
                          _('Unable to retrieve Nova availability zones.'))
        return []


class GeneralConfigAction(workflows.Action):
    nodegroup_name = forms.CharField(label=_("Template Name"))

    description = forms.CharField(label=_("Description"),
                                  required=False,
                                  widget=forms.Textarea(attrs={'rows': 4}))

    flavor = forms.ChoiceField(label=_("OpenStack Flavor"))

    availability_zone = forms.ChoiceField(
        label=_("Availability Zone"),
        help_text=_("Launch instances in this availability zone."),
        required=False,
        widget=forms.Select(attrs={"class": "availability_zone_field"})
    )

    storage = forms.ChoiceField(
        label=_("Attached storage location"),
        help_text=_("Choose a storage location"),
        choices=[],
        widget=forms.Select(attrs={
            "class": "storage_field switchable",
            'data-slug': 'storage_loc'
        }))

    volumes_per_node = forms.IntegerField(
        label=_("Volumes per node"),
        required=False,
        initial=1,
        widget=forms.TextInput(attrs={
            "class": "volume_per_node_field switched",
            "data-switch-on": "storage_loc",
            "data-storage_loc-cinder_volume": _('Volumes per node')
        })
    )

    volumes_size = forms.IntegerField(
        label=_("Volumes size (GB)"),
        required=False,
        initial=10,
        widget=forms.TextInput(attrs={
            "class": "volume_size_field switched",
            "data-switch-on": "storage_loc",
            "data-storage_loc-cinder_volume": _('Volumes size (GB)')
        })
    )

    volume_type = forms.ChoiceField(
        label=_("Volumes type"),
        required=False,
        widget=forms.Select(attrs={
            "class": "volume_type_field switched",
            "data-switch-on": "storage_loc",
            "data-storage_loc-cinder_volume": _('Volumes type')
        })
    )

    volume_local_to_instance = forms.BooleanField(
        label=_("Volume local to instance"),
        required=False,
        help_text=_("Instance and attached volumes will be created on the "
                    "same physical host"),
        widget=forms.CheckboxInput(attrs={
            "class": "volume_local_to_instance_field switched",
            "data-switch-on": "storage_loc",
            "data-storage_loc-cinder_volume": _('Volume local to instance')
        })
    )

    volumes_availability_zone = forms.ChoiceField(
        label=_("Volumes Availability Zone"),
        help_text=_("Create volumes in this availability zone."),
        required=False,
        widget=forms.Select(attrs={
            "class": "volumes_availability_zone_field switched",
            "data-switch-on": "storage_loc",
            "data-storage_loc-cinder_volume": _('Volumes Availability Zone')
        })
    )

    image = forms.DynamicChoiceField(label=_("Base Image"),
                                     required=False,
                                     add_item_link=BASE_IMAGE_URL)

    hidden_configure_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_configure_field"}))

    def __init__(self, request, *args, **kwargs):
        super(GeneralConfigAction, self).__init__(request, *args, **kwargs)

        hlps = helpers.Helpers(request)

        plugin, hadoop_version = (
            workflow_helpers.get_plugin_and_hadoop_version(request))

        if not saharaclient.SAHARA_FLOATING_IP_DISABLED:
            pools = neutron.floating_ip_pools_list(request)
            pool_choices = [(pool.id, pool.name) for pool in pools]
            pool_choices.insert(0, (None, "Do not assign floating IPs"))

            self.fields['floating_ip_pool'] = forms.ChoiceField(
                label=_("Floating IP Pool"),
                choices=pool_choices,
                required=False)

        self.fields["use_autoconfig"] = forms.BooleanField(
            label=_("Auto-configure"),
            help_text=_("If selected, instances of a node group will be "
                        "automatically configured during cluster "
                        "creation. Otherwise you should manually specify "
                        "configuration values."),
            required=False,
            widget=forms.CheckboxInput(),
            initial=True,
        )

        self.fields["proxygateway"] = forms.BooleanField(
            label=_("Proxy Gateway"),
            widget=forms.CheckboxInput(),
            help_text=_("Sahara will use instances of this node group to "
                        "access other cluster instances."),
            required=False)

        self.fields['is_public'] = acl_utils.get_is_public_form(
            _("node group template"))
        self.fields['is_protected'] = acl_utils.get_is_protected_form(
            _("node group template"))

        if saharaclient.VERSIONS.active == '2':
            self.fields['boot_storage'] = forms.ChoiceField(
                label=_("Boot storage location"),
                help_text=_("Choose a boot mode"),
                choices=storage_choices(request),
                widget=forms.Select(attrs={
                    "class": "boot_storage_field switchable",
                    'data-slug': 'boot_storage_loc'
                }))

            self.fields['boot_volume_type'] = forms.ChoiceField(
                label=_("Boot volume type"),
                required=False,
                widget=forms.Select(attrs={
                    "class": "boot_volume_type_field switched",
                    "data-switch-on": "boot_storage_loc",
                    "data-boot_storage_loc-cinder_volume":
                        _('Boot volume type')
                })
            )

            self.fields['boot_volume_local_to_instance'] = forms.BooleanField(
                label=_("Boot volume local to instance"),
                required=False,
                help_text=_("Boot volume locality"),
                widget=forms.CheckboxInput(attrs={
                    "class": "boot_volume_local_to_instance_field switched",
                    "data-switch-on": "boot_storage_loc",
                    "data-boot_storage_loc-cinder_volume":
                        _('Boot volume local to instance')
                })
            )

            self.fields['boot_volume_availability_zone'] = forms.ChoiceField(
                label=_("Boot volume availability Zone"),
                choices=self.populate_volumes_availability_zone_choices(
                    request, None),
                help_text=_("Create boot volume in this availability zone."),
                required=False,
                widget=forms.Select(attrs={
                    "class": "boot_volume_availability_zone_field switched",
                    "data-switch-on": "boot_storage_loc",
                    "data-boot_storage_loc-cinder_volume":
                        _('Boot volume availability zone')
                })
            )

        self.fields["plugin_name"] = forms.CharField(
            widget=forms.HiddenInput(),
            initial=plugin
        )
        self.fields["hadoop_version"] = forms.CharField(
            widget=forms.HiddenInput(),
            initial=hadoop_version
        )

        self.fields["storage"].choices = storage_choices(request)

        node_parameters = hlps.get_general_node_group_configs(plugin,
                                                              hadoop_version)
        for param in node_parameters:
            self.fields[param.name] = workflow_helpers.build_control(param)

        # when we copy or edit a node group template then
        # request contains valuable info in both GET and POST methods
        req = request.GET.copy()
        req.update(request.POST)
        if req.get("guide_template_type"):
            self.fields["guide_template_type"] = forms.CharField(
                required=False,
                widget=forms.HiddenInput(),
                initial=req.get("guide_template_type"))

        if is_cinder_enabled(request):
            volume_types = cinder.volume_type_list(request)
        else:
            volume_types = []

        self.fields['volume_type'].choices = [(None, _("No volume type"))] + \
                                             [(type.name, type.name)
                                              for type in volume_types]

        if saharaclient.VERSIONS.active == '2':
            self.fields['boot_volume_type'].choices = (
                self.fields['volume_type'].choices)

    def populate_flavor_choices(self, request, context):
        flavors = nova_utils.flavor_list(request)
        if flavors:
            return nova_utils.sort_flavor_list(request, flavors)
        return []

    def populate_availability_zone_choices(self, request, context):
        # The default is None, i.e. not specifying any availability zone
        az_list = [(None, _('No availability zone specified'))]
        az_list.extend([(az.zoneName, az.zoneName)
                        for az in instance_availability_zone_list(request)
                        if az.zoneState['available']])
        return az_list

    def populate_volumes_availability_zone_choices(self, request, context):
        az_list = [(None, _('No availability zone specified'))]
        if is_cinder_enabled(request):
            az_list.extend([(az.zoneName, az.zoneName)
                            for az in volume_availability_zone_list(
                                request) if az.zoneState['available']])
        return az_list

    def populate_image_choices(self, request, context):
        return workflow_helpers.populate_image_choices(self, request, context,
                                                       empty_choice=True)

    def get_help_text(self):
        extra = dict()
        plugin_name, hadoop_version = (
            workflow_helpers.get_plugin_and_hadoop_version(self.request))
        extra["plugin_name"] = plugin_name
        extra["hadoop_version"] = hadoop_version
        plugin = saharaclient.plugin_get_version_details(
            self.request, plugin_name, hadoop_version)
        extra["deprecated"] = workflow_helpers.is_version_of_plugin_deprecated(
            plugin, hadoop_version)
        return super(GeneralConfigAction, self).get_help_text(extra)

    class Meta(object):
        name = _("Configure Node Group Template")
        help_text_template = "nodegroup_templates/_configure_general_help.html"


class SecurityConfigAction(workflows.Action):
    def __init__(self, request, *args, **kwargs):
        super(SecurityConfigAction, self).__init__(request, *args, **kwargs)

        self.fields["security_autogroup"] = forms.BooleanField(
            label=_("Auto Security Group"),
            widget=forms.CheckboxInput(),
            help_text=_("Create security group for this Node Group."),
            required=False,
            initial=True)

        try:
            groups = neutron.security_group_list(request)
        except Exception:
            exceptions.handle(request,
                              _("Unable to get security group list."))
            raise

        security_group_list = [(sg.id, sg.name) for sg in groups]
        self.fields["security_groups"] = forms.MultipleChoiceField(
            label=_("Security Groups"),
            widget=forms.CheckboxSelectMultiple(),
            help_text=_("Launch instances in these security groups."),
            choices=security_group_list,
            required=False)

    class Meta(object):
        name = _("Security")
        help_text = _("Control access to instances of the node group.")


class CheckboxSelectMultiple(forms.CheckboxSelectMultiple):
    # TODO(shuyingya): should rewrite this class using the
    # new method "get_context" when Django version less than 1.11
    # no longer support.
    def build_attrs(self, extra_attrs=None, **kwargs):
        "Helper function for building an attribute dictionary."
        attrs = dict(self.attrs, **kwargs)
        if extra_attrs:
            attrs.update(extra_attrs)
        return attrs

    def render(self, name, value, attrs=None, choices=(), renderer=None):
        if value is None:
            value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)
        output = []
        initial_service = uuidutils.generate_uuid()
        str_values = set([encoding.force_text(v) for v in value])
        for i, (option_value, option_label) in enumerate(
                itertools.chain(self.choices, choices)):
            current_service = option_value.split(':')[0]
            if current_service != initial_service:
                if i > 0:
                    output.append("</ul>")
                service_description = _("%s processes: ") % current_service
                service_description = html.conditional_escape(
                    encoding.force_text(service_description))
                output.append("<label>%s</label>" % service_description)
                initial_service = current_service
                output.append(encoding.force_text("<ul>"))
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = ' for="%s"' % final_attrs['id']
            else:
                label_for = ''

            cb = forms.CheckboxInput(
                final_attrs, check_test=lambda value: value in str_values)
            option_value = encoding.force_text(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = html.conditional_escape(
                encoding.force_text(option_label))
            output.append(
                '<li><label%s>%s %s</label></li>' %
                (label_for, rendered_cb, option_label))
        output.append('</ul>')
        return safestring.mark_safe('\n'.join(output))


class SelectNodeProcessesAction(workflows.Action):
    def __init__(self, request, *args, **kwargs):
        super(SelectNodeProcessesAction, self).__init__(
            request, *args, **kwargs)

        plugin, hadoop_version = (
            workflow_helpers.get_plugin_and_hadoop_version(request))
        node_processes = {}
        try:
            version_details = saharaclient.plugin_get_version_details(
                request, plugin, hadoop_version)
            node_processes = version_details.node_processes
        except Exception:
            exceptions.handle(request,
                              _("Unable to generate process choices."))
        process_choices = []
        for service, processes in node_processes.items():
            for process in processes:
                choice_label = str(service) + ":" + str(process)
                process_choices.append((choice_label, process))

        self.fields["processes"] = forms.MultipleChoiceField(
            label=_("Select Node Group Processes"),
            widget=CheckboxSelectMultiple(),
            choices=process_choices,
            required=True)

    class Meta(object):
        name = _("Node Processes")
        help_text = _("Select node processes for the node group")


class SelectNodeGroupSharesAction(workflows.Action):
    def __init__(self, request, *args, **kwargs):
        super(SelectNodeGroupSharesAction, self).__init__(
            request, *args, **kwargs)

        possible_shares = self.get_possible_shares(request)

        self.fields["shares"] = workflow_helpers.MultipleShareChoiceField(
            label=_("Select Shares"),
            widget=workflow_helpers.ShareWidget(choices=possible_shares),
            required=False,
            choices=possible_shares
        )

    def get_possible_shares(self, request):
        try:
            shares = manilaclient.share_list(request)
            choices = [(s.id, s.name) for s in shares]
        except Exception:
            exceptions.handle(request, _("Failed to get list of shares"))
            choices = []
        return choices

    class Meta(object):
        name = _("Shares")
        help_text = _("Select the manila shares for this node group")


class GeneralConfig(workflows.Step):
    action_class = GeneralConfigAction
    contributes = ("general_nodegroup_name", )

    def contribute(self, data, context):
        for k, v in data.items():
            if "hidden" in k:
                continue
            context["general_" + k] = v if v != "None" else None
        return context


class SecurityConfig(workflows.Step):
    action_class = SecurityConfigAction
    contributes = ("security_autogroup", "security_groups")


class SelectNodeProcesses(workflows.Step):
    action_class = SelectNodeProcessesAction

    def contribute(self, data, context):
        post = self.workflow.request.POST
        context['general_processes'] = post.getlist('processes')
        return context


class SelectNodeGroupShares(workflows.Step):
    action_class = SelectNodeGroupSharesAction

    def contribute(self, data, context):
        post = self.workflow.request.POST
        shares_details = []
        for index in range(0, len(self.action.fields['shares'].choices) * 3):
            if index % 3 == 0:
                share = post.get("shares_{0}".format(index))
                if share:
                    path = post.get("shares_{0}".format(index + 1))
                    permissions = post.get("shares_{0}".format(index + 2))
                    shares_details.append({
                        "id": share,
                        "path": path,
                        "access_level": permissions
                    })
        context['ngt_shares'] = shares_details
        return context


class ConfigureNodegroupTemplate(workflow_helpers.ServiceParametersWorkflow,
                                 workflow_helpers.StatusFormatMixin):
    slug = "configure_nodegroup_template"
    name = _("Create Node Group Template")
    finalize_button_name = _("Create")
    success_message = _("Created Node Group Template %s")
    name_property = "general_nodegroup_name"
    success_url = ("horizon:project:data_processing.clusters:"
                   "nodegroup-templates-tab")
    default_steps = (GeneralConfig, SelectNodeProcesses, SecurityConfig, )

    def __init__(self, request, context_seed, entry_point, *args, **kwargs):
        hlps = helpers.Helpers(request)

        plugin, hadoop_version = (
            workflow_helpers.get_plugin_and_hadoop_version(request))

        general_parameters, service_parameters = \
            hlps.get_general_and_service_nodegroups_parameters(plugin,
                                                               hadoop_version)

        if saharaclient.base.is_service_enabled(request, 'share'):
            ConfigureNodegroupTemplate._register_step(self,
                                                      SelectNodeGroupShares)

        self._populate_tabs(general_parameters, service_parameters)

        super(ConfigureNodegroupTemplate, self).__init__(request,
                                                         context_seed,
                                                         entry_point,
                                                         *args, **kwargs)

    def is_valid(self):
        missing = self.depends_on - set(self.context.keys())
        if missing:
            raise exceptions.WorkflowValidationError(
                "Unable to complete the workflow. The values %s are "
                "required but not present." % ", ".join(missing))
        checked_steps = []

        if "general_processes" in self.context:
            checked_steps = self.context["general_processes"]
        enabled_services = set([])
        for process_name in checked_steps:
            enabled_services.add(str(process_name).split(":")[0])

        steps_valid = True
        for step in self.steps:
            process_name = str(getattr(step, "process_name", None))
            if process_name not in enabled_services and \
                    not isinstance(step, GeneralConfig):
                continue
            if not step.action.is_valid():
                steps_valid = False
                step.has_errors = True
        if not steps_valid:
            return steps_valid
        return self.validate(self.context)

    def handle(self, request, context):
        try:
            processes = []
            for service_process in context["general_processes"]:
                processes.append(str(service_process).split(":")[1])

            configs_dict = (
                workflow_helpers.parse_configs_from_context(
                    context, self.defaults))

            plugin, hadoop_version = (
                workflow_helpers.get_plugin_and_hadoop_version(request))

            volumes_per_node = None
            volumes_size = None
            volumes_availability_zone = None
            volume_type = None
            volume_local_to_instance = False

            if context["general_storage"] == "cinder_volume":
                volumes_per_node = context["general_volumes_per_node"]
                volumes_size = context["general_volumes_size"]
                volumes_availability_zone = \
                    context["general_volumes_availability_zone"]
                volume_type = context["general_volume_type"]
                volume_local_to_instance = \
                    context["general_volume_local_to_instance"]

            ngt_shares = context.get('ngt_shares', [])

            image_id = context["general_image"] or None

            if (saharaclient.VERSIONS.active == '2'
                    and context["general_boot_storage"] == "cinder_volume"):
                boot_from_volume = True
                boot_volume_type = (
                    context["general_boot_volume_type"] or None
                )
                boot_volume_availability_zone = (
                    context["general_boot_volume_availability_zone"] or None
                )
                boot_volume_local_to_instance = (
                    context["general_boot_volume_local_to_instance"]
                )
            else:
                boot_from_volume = None
                boot_volume_type = None
                boot_volume_availability_zone = None
                boot_volume_local_to_instance = None

            ngt = saharaclient.nodegroup_template_create(
                request,
                name=context["general_nodegroup_name"],
                plugin_name=plugin,
                hadoop_version=hadoop_version,
                description=context["general_description"],
                flavor_id=context["general_flavor"],
                volumes_per_node=volumes_per_node,
                volumes_size=volumes_size,
                volumes_availability_zone=volumes_availability_zone,
                volume_type=volume_type,
                volume_local_to_instance=volume_local_to_instance,
                node_processes=processes,
                node_configs=configs_dict,
                floating_ip_pool=context.get("general_floating_ip_pool"),
                security_groups=context["security_groups"],
                auto_security_group=context["security_autogroup"],
                is_proxy_gateway=context["general_proxygateway"],
                availability_zone=context["general_availability_zone"],
                use_autoconfig=context['general_use_autoconfig'],
                shares=ngt_shares,
                is_public=context['general_is_public'],
                is_protected=context['general_is_protected'],
                boot_from_volume=boot_from_volume,
                boot_volume_type=boot_volume_type,
                boot_volume_availability_zone=boot_volume_availability_zone,
                boot_volume_local_to_instance=boot_volume_local_to_instance,
                image_id=image_id)

            hlps = helpers.Helpers(request)
            if hlps.is_from_guide():
                guide_type = context["general_guide_template_type"]
                request.session[guide_type + "_name"] = (
                    context["general_nodegroup_name"])
                request.session[guide_type + "_id"] = ngt.id
                self.success_url = (
                    "horizon:project:data_processing.clusters:cluster_guide")

            return True
        except api_base.APIException as e:
            self.error_description = str(e)
            return False
        except Exception:
            exceptions.handle(request)


class SelectPluginAction(workflows.Action,
                         workflow_helpers.PluginAndVersionMixin):
    hidden_create_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_create_field"}))

    def __init__(self, request, *args, **kwargs):
        super(SelectPluginAction, self).__init__(request, *args, **kwargs)

        sahara = saharaclient.client(request)
        self._generate_plugin_version_fields(sahara)

    class Meta(object):
        name = _("Select plugin and hadoop version")
        help_text_template = "nodegroup_templates/_create_general_help.html"


class SelectPlugin(workflows.Step):
    action_class = SelectPluginAction
    contributes = ("plugin_name", "hadoop_version")

    def contribute(self, data, context):
        context = super(SelectPlugin, self).contribute(data, context)
        context["plugin_name"] = data.get('plugin_name', None)
        context["hadoop_version"] = \
            data.get(context["plugin_name"] + "_version", None)
        return context


class CreateNodegroupTemplate(workflows.Workflow):
    slug = "create_nodegroup_template"
    name = _("Create Node Group Template")
    finalize_button_name = _("Next")
    success_message = _("Created")
    failure_message = _("Could not create")
    success_url = ("horizon:project:data_processing.clusters:"
                   "nodegroup-templates-tab")
    default_steps = (SelectPlugin,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import messages
from openstack_dashboard.test.integration_tests.regions import tables


class BaseDataProcessingPage(basepage.BaseNavigationPage):

    TABLE_NAME = ""

    # index of name column in binary jobs table
    TABLE_NAME_COLUMN = 'name'

    def __new__(cls, *args, **kwargs):
        table_bases = (tables.TableRegion,) + cls.get_table_mixins()
        cls.TableRegion = type('PageTableRegion', table_bases,
                               {'name': cls.TABLE_NAME})
        return super(BaseDataProcessingPage, cls).__new__(cls, *args, **kwargs)

    @classmethod
    def get_table_mixins(cls):
        return ()

    def _get_row_with_name(self, name, name_column=TABLE_NAME_COLUMN):
        try:
            return self.table.get_row(name_column, name)
        except Exception:
            return None

    @property
    def table(self):
        return self.TableRegion(self.driver, self.conf)

    def is_present(self, name):
        return bool(self._get_row_with_name(name))

    def has_success_message(self):
        return self.find_message_and_dismiss(messages.SUCCESS)

    def has_error_message(self):
        return self.find_message_and_dismiss(messages.ERROR)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from horizon import exceptions
from horizon import forms
from horizon import workflows

from django.utils.translation import gettext_lazy as _
from saharaclient.api import base as api_base

from openstack_dashboard.api import nova

from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
from sahara_dashboard.content.data_processing.utils import neutron_support
import sahara_dashboard.content.data_processing.utils. \
    workflow_helpers as whelpers
from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing.clusters. \
    cluster_templates.workflows.create as t_flows


KEYPAIR_IMPORT_URL = "horizon:project:key_pairs:import"
BASE_IMAGE_URL = "horizon:project:data_processing.clusters:register"
TEMPLATE_UPLOAD_URL = (
    "horizon:project:data_processing.clusters:upload_file")


class SelectPluginAction(t_flows.SelectPluginAction):
    class Meta(object):
        name = _("Select plugin and hadoop version for cluster")
        help_text_template = "clusters/_create_general_help.html"


class SelectPlugin(t_flows.SelectPlugin):
    action_class = SelectPluginAction


class CreateCluster(t_flows.CreateClusterTemplate):
    slug = "create_cluster"
    name = _("Launch Cluster")
    success_url = "horizon:project:data_processing.clusters:clusters-tab"
    default_steps = (SelectPlugin,)


class GeneralConfigAction(workflows.Action):
    populate_neutron_management_network_choices = \
        neutron_support.populate_neutron_management_network_choices

    hidden_configure_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_configure_field"}))

    hidden_to_delete_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_to_delete_field"}))

    cluster_name = forms.CharField(label=_("Cluster Name"))

    description = forms.CharField(label=_("Description"),
                                  required=False,
                                  widget=forms.Textarea(attrs={'rows': 4}))
    cluster_template = forms.DynamicChoiceField(
        label=_("Cluster Template"),
        initial=(None, "None"),
        add_item_link=TEMPLATE_UPLOAD_URL)

    cluster_count = forms.IntegerField(min_value=1,
                                       label=_("Cluster Count"),
                                       initial=1,
                                       help_text=(
                                           _("Number of clusters to launch.")))

    image = forms.DynamicChoiceField(label=_("Base Image"),
                                     add_item_link=BASE_IMAGE_URL)

    keypair = forms.DynamicChoiceField(
        label=_("Keypair"),
        required=False,
        help_text=_("Which keypair to use for authentication."),
        add_item_link=KEYPAIR_IMPORT_URL)

    def __init__(self, request, *args, **kwargs):
        super(GeneralConfigAction, self).__init__(request, *args, **kwargs)

        plugin, hadoop_version = whelpers.\
            get_plugin_and_hadoop_version(request)

        if saharaclient.base.is_service_enabled(request, 'network'):
            self.fields["neutron_management_network"] = forms.ChoiceField(
                label=_("Neutron Management Network"),
                choices=self.populate_neutron_management_network_choices(
                    request, {})
            )

        self.fields['is_public'] = acl_utils.get_is_public_form(
            _("cluster"))
        self.fields['is_protected'] = acl_utils.get_is_protected_form(
            _("cluster"))

        self.fields["plugin_name"] = forms.CharField(
            widget=forms.HiddenInput(),
            initial=plugin
        )
        self.fields["hadoop_version"] = forms.CharField(
            widget=forms.HiddenInput(),
            initial=hadoop_version
        )

    def populate_image_choices(self, request, context):
        return whelpers.populate_image_choices(self, request, context)

    def populate_keypair_choices(self, request, context):
        try:
            keypairs = nova.keypair_list(request)
        except Exception:
            keypairs = []
            exceptions.handle(request,
                              _("Unable to fetch keypair choices."))
        keypair_list = [(kp.name, kp.name) for kp in keypairs]
        keypair_list.insert(0, ("", _("No keypair")))

        return keypair_list

    def populate_cluster_template_choices(self, request, context):
        templates = saharaclient.cluster_template_list(request)

        plugin, hadoop_version = whelpers.\
            get_plugin_and_hadoop_version(request)

        choices = []
        for template in templates:
            version = (
                getattr(template, "hadoop_version", None) or
                template.plugin_version
            )
            if version == hadoop_version and template.plugin_name == plugin:
                choices.append((template.id, template.name))

        if not choices:
            choices.append(("", _("No Templates Available")))
        # cluster_template_id comes from cluster templates table, when
        # Create Cluster from template is clicked there
        selected_template_name = None
        req = request.GET or request.POST
        if req.get("cluster_template_name"):
            selected_template_name = (
                req.get("cluster_template_name"))
        if selected_template_name:
            for template in templates:
                if template.name == selected_template_name:
                    selected_template_id = template.id
                    break
        else:
            selected_template_id = (
                req.get("cluster_template_id", None))

        for template in templates:
            if template.id == selected_template_id:
                self.fields['cluster_template'].initial = template.id

        return choices

    def get_help_text(self):
        extra = dict()
        plugin, hadoop_version = whelpers.\
            get_plugin_and_hadoop_version(self.request)
        extra["plugin_name"] = plugin
        extra["hadoop_version"] = hadoop_version
        return super(GeneralConfigAction, self).get_help_text(extra)

    def clean(self):
        cleaned_data = super(GeneralConfigAction, self).clean()
        if cleaned_data.get("hidden_configure_field", None) \
                == "create_nodegroup":
            self._errors = dict()
        return cleaned_data

    class Meta(object):
        name = _("Configure Cluster")
        help_text_template = "clusters/_configure_general_help.html"


class GeneralConfig(workflows.Step):
    action_class = GeneralConfigAction
    contributes = ("hidden_configure_field", )

    def contribute(self, data, context):
        for k, v in data.items():
            context["general_" + k] = v

        return context


class ConfigureCluster(whelpers.StatusFormatMixin, workflows.Workflow):
    slug = "configure_cluster"
    name = _("Launch Cluster")
    finalize_button_name = _("Launch")
    success_message = _("Launched Cluster %s")
    name_property = "general_cluster_name"
    success_url = "horizon:project:data_processing.clusters:clusters-tab"
    default_steps = (GeneralConfig, )

    def handle(self, request, context):
        try:
            # TODO(nkonovalov) Implement AJAX Node Groups.
            node_groups = None

            plugin, hadoop_version = whelpers.\
                get_plugin_and_hadoop_version(request)

            cluster_template_id = context["general_cluster_template"] or None
            user_keypair = context["general_keypair"] or None
            image_id = context["general_image"] or None
            saharaclient.cluster_create(
                request,
                context["general_cluster_name"],
                plugin, hadoop_version,
                cluster_template_id=cluster_template_id,
                default_image_id=image_id,
                description=context["general_description"],
                node_groups=node_groups,
                user_keypair_id=user_keypair,
                count=context['general_cluster_count'],
                net_id=context.get("general_neutron_management_network", None),
                is_public=context['general_is_public'],
                is_protected=context['general_is_protected']
            )
            return True
        except api_base.APIException as e:
            self.error_description = str(e)
            return False
        except Exception:
            exceptions.handle(request,
                              _('Unable to create the cluster'))
            return False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.forms import fields
from horizon import workflows

from sahara_dashboard.content.data_processing \
    .utils import helpers
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
import sahara_dashboard.content.data_processing \
    .utils.workflow_helpers as whelpers
from sahara_dashboard.api import sahara as saharaclient


JOB_BINARY_CREATE_URL = ("horizon:project:data_processing.jobs"
                         ":create-job-binary")


class AdditionalLibsAction(workflows.Action):

    lib_binaries = forms.DynamicChoiceField(
        label=_("Choose libraries"),
        required=False,
        add_item_link=JOB_BINARY_CREATE_URL,
        widget=forms.Select(
            attrs={
                'class': 'switched',
                'data-switch-on': 'jobtype',
                'data-jobtype-pig': _("Choose libraries"),
                'data-jobtype-hive': _("Choose libraries"),
                'data-jobtype-shell': _("Choose additional files"),
                'data-jobtype-spark': _("Choose libraries"),
                'data-jobtype-java': _("Choose libraries"),
                'data-jobtype-mapreduce': _("Choose libraries"),
                'data-jobtype-mapreduce.streaming': _("Choose libraries")
            }))

    lib_ids = forms.CharField(
        required=False,
        widget=forms.HiddenInput())

    def populate_lib_binaries_choices(self, request, context):
        job_binaries = saharaclient.job_binary_list(request)

        choices = [(job_binary.id, job_binary.name)
                   for job_binary in job_binaries]
        choices.insert(0, ('', _("-- not selected --")))

        return choices

    class Meta(object):
        name = _("Libs")
        help_text_template = "job_templates/_create_job_libs_help.html"


class GeneralConfigAction(workflows.Action):
    job_name = forms.CharField(label=_("Name"))

    job_type = forms.ChoiceField(label=_("Job Type"),
                                 widget=forms.Select(attrs={
                                     'class': 'switchable',
                                     'data-slug': 'jobtype'
                                 }))

    main_binary = forms.DynamicChoiceField(
        label=_("Choose a main binary"),
        required=False,
        help_text=_("Choose the binary which "
                    "should be used in this Job."),
        add_item_link=JOB_BINARY_CREATE_URL,
        widget=fields.DynamicSelectWidget(
            attrs={
                'class': 'switched',
                'data-switch-on': 'jobtype',
                'data-jobtype-pig': _("Choose a main binary"),
                'data-jobtype-hive': _("Choose a main binary"),
                'data-jobtype-shell': _("Choose a shell script"),
                'data-jobtype-spark': _("Choose a main binary"),
                'data-jobtype-storm': _("Choose a main binary"),
                'data-jobtype-storm.pyleus': _("Choose a main binary"),
                'data-jobtype-mapreduce.streaming': _("Choose a main binary")
            }))

    job_description = forms.CharField(label=_("Description"),
                                      required=False,
                                      widget=forms.Textarea(attrs={'rows': 4}))
    is_public = acl_utils.get_is_public_form(_("job"))
    is_protected = acl_utils.get_is_protected_form(_("job"))

    def __init__(self, request, context, *args, **kwargs):
        super(GeneralConfigAction,
              self).__init__(request, context, *args, **kwargs)
        req = request.GET or request.POST
        if req.get("guide_job_type"):
            self.fields["job_type"].initial = (
                req.get("guide_job_type").lower())

    def populate_job_type_choices(self, request, context):
        choices = []
        choices_list = saharaclient.job_types_list(request)

        for choice in choices_list:
            job_type = choice.name.lower()
            if job_type in helpers.JOB_TYPE_MAP:
                choices.append((job_type, helpers.JOB_TYPE_MAP[job_type][0]))
        return choices

    def populate_main_binary_choices(self, request, context):
        job_binaries = saharaclient.job_binary_list(request)

        choices = [(job_binary.id, job_binary.name)
                   for job_binary in job_binaries]
        choices.insert(0, ('', _("-- not selected --")))
        return choices

    def clean(self):
        cleaned_data = super(workflows.Action, self).clean()
        job_type = cleaned_data.get("job_type", "")

        if job_type in ["Java", "MapReduce"]:
            cleaned_data['main_binary'] = None

        return cleaned_data

    class Meta(object):
        name = _("Create Job Template")
        help_text_template = "job_templates/_create_job_help.html"


class ConfigureInterfaceArgumentsAction(workflows.Action):
    hidden_arguments_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_arguments_field"}))
    argument_ids = forms.CharField(
        required=False,
        widget=forms.HiddenInput())

    def __init__(self, request, *args, **kwargs):
        super(ConfigureInterfaceArgumentsAction, self).__init__(
            request, *args, **kwargs)
        req = request.GET or request.POST
        if 'argument_ids' in req:
            self.arguments = []
            for id in json.loads(req['argument_ids']):
                fields = {
                    "name": "argument_name_" + str(id),
                    "description": "argument_description_" + str(id),
                    "mapping_type": "argument_mapping_type_" + str(id),
                    "location": "argument_location_" + str(id),
                    "value_type": "argument_value_type_" + str(id),
                    "default_value": "argument_default_value_" + str(id)}
                argument = {k: req[v]
                            for k, v in fields.items()}
                required_field = "argument_required_" + str(id)
                fields.update({"required": required_field})
                argument.update(
                    {"required": required_field in req})
                self.arguments.append(argument)

                whelpers.build_interface_argument_fields(self, **fields)

    def clean(self):
        cleaned_data = super(ConfigureInterfaceArgumentsAction, self).clean()
        return cleaned_data

    class Meta(object):
        name = _("Interface Arguments")


class ConfigureArguments(workflows.Step):
    action_class = ConfigureInterfaceArgumentsAction
    contributes = ("hidden_arguments_field", )
    template_name = "job_templates/job_interface_arguments_template.html"

    def contribute(self, data, context):
        for k, v in data.items():
            context[k] = v
        return context


class GeneralConfig(workflows.Step):
    action_class = GeneralConfigAction
    contributes = ("job_name", "job_type", "job_description", "main_binary")

    def contribute(self, data, context):
        for k, v in data.items():
            if k == "job_type":
                context[k] = helpers.JOB_TYPE_MAP[v][1]
            else:
                context[k] = v
        return context


class ConfigureLibs(workflows.Step):
    action_class = AdditionalLibsAction
    template_name = "job_templates/library_template.html"

    def contribute(self, data, context):
        chosen_libs = json.loads(data.get("lib_ids", '[]'))
        for index, library in enumerate(chosen_libs):
            context["lib_%s" % index] = library
        return context


class CreateJob(workflows.Workflow):
    slug = "create_job"
    name = _("Create Job Template")
    finalize_button_name = _("Create")
    success_message = _("Job created")
    failure_message = _("Could not create job template")
    success_url = "horizon:project:data_processing.jobs:index"
    default_steps = (GeneralConfig, ConfigureLibs, ConfigureArguments)

    def handle(self, request, context):
        main_locations = []
        lib_locations = []

        for k in context.keys():
            if k.startswith('lib_'):
                lib_locations.append(context.get(k))

        if context.get("main_binary", None):
            job_type = context.get("job_type", None)
            if job_type not in ["Java", "MapReduce"]:
                main_locations.append(context["main_binary"])

        argument_ids = json.loads(context['argument_ids'])
        interface = [
            {
                "name": context['argument_name_' + str(arg_id)],
                "description": (context['argument_description_' + str(arg_id)]
                                or None),
                "mapping_type": context['argument_mapping_type_'
                                        + str(arg_id)],
                "location": context['argument_location_' + str(arg_id)],
                "value_type": context['argument_value_type_' + str(arg_id)],
                "required": context['argument_required_' + str(arg_id)],
                "default": (context['argument_default_value_' + str(arg_id)]
                            or None)
            } for arg_id in argument_ids
        ]

        try:
            job = saharaclient.job_create(
                request,
                context["job_name"],
                context["job_type"],
                main_locations,
                lib_locations,
                context["job_description"],
                interface=interface,
                is_public=context['is_public'],
                is_protected=context['is_protected']
            )

            hlps = helpers.Helpers(request)
            if hlps.is_from_guide():
                request.session["guide_job_id"] = job.id
                request.session["guide_job_type"] = context["job_type"]
                request.session["guide_job_name"] = context["job_name"]
                self.success_url = (
                    "horizon:project:data_processing.wizard:jobex_guide")
            return True
        except Exception:
            exceptions.handle(request)
            return False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from openstack_dashboard.api import glance

from sahara_dashboard.api import sahara as saharaclient


class ImageForm(forms.SelfHandlingForm):
    image_id = forms.CharField(widget=forms.HiddenInput())
    tags_list = forms.CharField(widget=forms.HiddenInput())
    user_name = forms.CharField(max_length=80, label=_("User Name"))
    description = forms.CharField(max_length=80,
                                  label=_("Description"),
                                  required=False,
                                  widget=forms.Textarea(attrs={'rows': 4}))

    def handle(self, request, data):
        try:
            image_id = data['image_id']
            user_name = data['user_name']
            desc = data['description']
            saharaclient.image_update(request, image_id, user_name, desc)

            image_tags = json.loads(data["tags_list"])
            saharaclient.image_tags_update(request, image_id, image_tags)
            updated_image = saharaclient.image_get(request, image_id)
            messages.success(request,
                             _("Successfully updated image."))
            return updated_image
        except Exception:
            exceptions.handle(request,
                              _("Failed to update image."))
            return False


class EditTagsForm(ImageForm):
    image_id = forms.CharField(widget=forms.HiddenInput())


class RegisterImageForm(ImageForm):
    image_id = forms.ChoiceField(label=_("Image"))

    def __init__(self, request, *args, **kwargs):
        super(RegisterImageForm, self).__init__(request, *args, **kwargs)
        self._populate_image_id_choices()

    def _populate_image_id_choices(self):
        images = self._get_available_images(self.request)
        choices = [(image.id, image.name)
                   for image in images
                   if image.to_dict()['properties'].get(
                       "image_type") != "snapshot"]
        if choices:
            choices.insert(0, ("", _("Select Image")))
        else:
            choices.insert(0, ("", _("No images available.")))
        self.fields['image_id'].choices = choices

    def _get_images(self, request, filter):
        try:
            images, _more, _prev = (
                glance.image_list_detailed(request, filters=filter))
        except Exception:
            images = []
            exceptions.handle(request,
                              _("Unable to retrieve images with filter %s.") %
                              filter)
        return images

    def _get_public_images(self, request):
        filter = {"is_public": True,
                  "status": "active"}
        return self._get_images(request, filter)

    def _get_tenant_images(self, request):
        filter = {"owner": request.user.tenant_id,
                  "status": "active"}
        return self._get_images(request, filter)

    def _get_available_images(self, request):

        images = self._get_tenant_images(request)
        if request.user.is_superuser:
            images += self._get_public_images(request)

        final_images = []

        try:
            image_ids = set(img.id for img in saharaclient.image_list(request))
        except Exception:
            image_ids = set()
            exceptions.handle(request,
                              _("Unable to fetch available images."))

        for image in images:
            if (image not in final_images and
                    image.id not in image_ids and
                    image.container_format not in ('aki', 'ari')):
                final_images.append(image)
        return final_images



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

from sahara_dashboard.api import manila as manilaclient
from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
from sahara_dashboard.content.data_processing \
    .utils import helpers


class GeneralConfigAction(workflows.Action):
    data_source_name = forms.CharField(label=_("Name"))

    data_source_type = forms.ChoiceField(
        label=_("Data Source Type"),
        widget=forms.Select(attrs={
            "class": "switchable",
            "data-slug": "ds_type"
        }))

    data_source_manila_share = forms.ChoiceField(
        label=_("Manila share"),
        required=False,
        widget=forms.Select(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-manila": _("Manila share")
        }))

    data_source_url = forms.CharField(
        label=_("URL"),
        widget=forms.TextInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-manila": _("Path on share"),
            "data-ds_type-swift": _("URL"),
            "data-ds_type-hdfs": _("URL"),
            "data-ds_type-maprfs": _("URL"),
            "data-ds_type-s3": _("URL"),
        }))

    data_source_credential_user = forms.CharField(
        label=_("Source username"),
        required=False,
        widget=forms.TextInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-swift": _("Source username")
        }))

    data_source_credential_pass = forms.CharField(
        widget=forms.PasswordInput(attrs={
            'class': 'switched',
            'data-switch-on': 'ds_type',
            'data-ds_type-swift': _("Source password"),
            'autocomplete': 'off'
        }),
        label=_("Source password"),
        required=False)

    data_source_credential_accesskey = forms.CharField(
        label=_("S3 accsss key"),
        required=False,
        widget=forms.TextInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-s3": _("S3 access key")
        }))

    data_source_credential_secretkey = forms.CharField(
        widget=forms.PasswordInput(attrs={
            'class': 'switched',
            'data-switch-on': 'ds_type',
            'data-ds_type-s3': _("S3 secret key"),
            'autocomplete': 'off'
        }),
        label=_("S3 secret key"),
        required=False)

    data_source_credential_endpoint = forms.CharField(
        label=_("S3 endpoint"),
        required=False,
        help_text=_("Endpoint should be specified without protocol"),
        widget=forms.TextInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-s3": _("S3 endpoint")
        }))

    data_source_credential_s3_ssl = forms.BooleanField(
        label=_("Use SSL"),
        required=False,
        initial=False,
        widget=forms.CheckboxInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-s3": _("Use SSL")
        }))

    data_source_credential_s3_bucket_in_path = forms.BooleanField(
        label=_("Use bucket in path"),
        required=False,
        initial=True,
        help_text=_("If checked, bucket will be in path instead of host"),
        widget=forms.CheckboxInput(attrs={
            "class": "switched",
            "data-switch-on": "ds_type",
            "data-ds_type-s3": _("Use bucket in path")
        }))

    data_source_description = forms.CharField(
        label=_("Description"),
        required=False,
        widget=forms.Textarea(attrs={'rows': 4}))

    is_public = acl_utils.get_is_public_form(_("data source"))
    is_protected = acl_utils.get_is_protected_form(_("data source"))

    def __init__(self, request, *args, **kwargs):
        super(GeneralConfigAction, self).__init__(request, *args, **kwargs)

        self.fields["data_source_type"].choices = [("swift", "Swift"),
                                                   ("hdfs", "HDFS"),
                                                   ("maprfs", "MapR FS"),
                                                   ("s3", "S3")]
        # If Manila is running, enable it as a choice for a data source
        if saharaclient.base.is_service_enabled(request, 'share'):
            self.fields["data_source_type"].choices.append(
                ("manila", "Manila"))
            self.fields["data_source_manila_share"].choices = (
                self.populate_manila_share_choices(request)
            )

    def populate_manila_share_choices(self, request):
        try:
            shares = manilaclient.share_list(request)
            choices = [(s.id, s.name) for s in shares]
        except Exception:
            exceptions.handle(request, _("Failed to get list of shares"))
            choices = []
        return choices

    class Meta(object):
        name = _("Create Data Source")
        help_text_template = "data_sources/_create_data_source_help.html"


class GeneralConfig(workflows.Step):
    action_class = GeneralConfigAction

    def contribute(self, data, context):
        for k, v in data.items():
            context["general_" + k] = v

        context["source_url"] = context["general_data_source_url"]
        ds_type = context["general_data_source_type"]

        if ds_type == "swift":
            if not context["general_data_source_url"].startswith("swift://"):
                context["source_url"] = "swift://{0}".format(
                    context["general_data_source_url"])
        elif ds_type == "manila":
            context["source_url"] = "{0}://{1}{2}".format(
                ds_type,
                context["general_data_source_manila_share"],
                context["general_data_source_url"])
        return context


class CreateDataSource(workflows.Workflow):
    slug = "create_data_source"
    name = _("Create Data Source")
    finalize_button_name = _("Create")
    success_message = _("Data source created")
    failure_message = _("Could not create data source")
    success_url = "horizon:project:data_processing.jobs:index"
    default_steps = (GeneralConfig, )

    def handle(self, request, context):
        s3_credentials = {}
        if context["general_data_source_type"] == "s3":
            if context.get("general_data_source_credential_accesskey", None):
                s3_credentials["accesskey"] = context[
                    "general_data_source_credential_accesskey"]
            if context.get("general_data_source_credential_secretkey", None):
                s3_credentials["secretkey"] = context[
                    "general_data_source_credential_secretkey"]
            if context.get("general_data_source_credential_endpoint", None):
                s3_credentials["endpoint"] = context[
                    "general_data_source_credential_endpoint"]
            s3_credentials["bucket_in_path"] = context[
                "general_data_source_credential_s3_bucket_in_path"]
            s3_credentials["ssl"] = context[
                "general_data_source_credential_s3_ssl"]
        s3_credentials = s3_credentials or None
        try:
            self.object = saharaclient.data_source_create(
                request,
                context["general_data_source_name"],
                context["general_data_source_description"],
                context["general_data_source_type"],
                context["source_url"],
                context.get("general_data_source_credential_user", None),
                context.get("general_data_source_credential_pass", None),
                is_public=context['general_is_public'],
                is_protected=context['general_is_protected'],
                s3_credentials=s3_credentials
            )

            hlps = helpers.Helpers(request)
            if hlps.is_from_guide():
                request.session["guide_datasource_id"] = self.object.id
                request.session["guide_datasource_name"] = self.object.name
                self.success_url = (
                    "horizon:project:data_processing.wizard:jobex_guide")
            return True
        except Exception:
            exceptions.handle(request)
            return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from django.urls import reverse
from oslo_serialization import jsonutils

from openstack_dashboard import api as dash_api

from sahara_dashboard import api
from sahara_dashboard.test import helpers as test
from sahara_dashboard import utils


INDEX_URL = reverse('horizon:project:data_processing.clusters:'
                    'cluster-templates-tab')
DETAILS_URL = reverse(
    'horizon:project:data_processing.clusters:ct-details', args=['id'])


class DataProcessingClusterTemplateTests(test.TestCase):

    @test.create_mocks({api.sahara: ('cluster_template_list',
                                     'image_list',
                                     'cluster_list',
                                     'nodegroup_template_list')})
    def test_index(self):
        self.mock_cluster_template_list.return_value = \
            self.cluster_templates.list()
        res = self.client.get(INDEX_URL)
        self.mock_cluster_template_list.assert_called_once_with(
            test.IsHttpRequest(), {})
        self.assertTemplateUsed(res, 'clusters/index.html')
        self.assertContains(res, 'Cluster Templates')
        self.assertContains(res, 'Name')

    @test.create_mocks({api.sahara: ('cluster_template_get',
                                     'nodegroup_template_get'),
                        dash_api.nova: ('flavor_get',)})
    def test_details(self):
        flavor = self.flavors.first()
        ct = self.cluster_templates.first()
        self.mock_flavor_get.return_value = flavor
        self.mock_cluster_template_get.return_value = ct
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')

    @test.create_mocks({api.sahara: ('client',
                                     'cluster_template_get',
                                     'plugin_get_version_details',
                                     'nodegroup_template_find')})
    def test_copy(self):
        ct = self.cluster_templates.first()
        ngts = self.nodegroup_templates.list()
        configs = self.plugins_configs.first()
        self.mock_cluster_template_get.return_value = ct
        self.mock_plugin_get_version_details.return_value = configs
        self.mock_nodegroup_template_find.return_value = ngts

        url = reverse('horizon:project:data_processing.clusters:ct-copy',
                      args=[ct.id])
        res = self.client.get(url)
        self.mock_cluster_template_get.assert_called_once_with(
            test.IsHttpRequest(), ct.id)
        workflow = res.context['workflow']
        step = workflow.get_step("generalconfigaction")
        self.assertEqual(step.action['cluster_template_name'].field.initial,
                         ct.name + "-copy")

    @test.create_mocks({api.sahara: ('cluster_template_list',
                                     'cluster_template_delete')})
    def test_delete(self):
        ct = self.cluster_templates.first()
        self.mock_cluster_template_list.return_value = \
            self.cluster_templates.list()
        self.mock_cluster_template_delete.return_value = None

        form_data = {'action': 'cluster_templates__delete__%s' % ct.id}
        res = self.client.post(INDEX_URL, form_data)

        self.mock_cluster_template_list.assert_called_once_with(
            test.IsHttpRequest(), {})
        self.mock_cluster_template_delete.assert_called_once_with(
            test.IsHttpRequest(), ct.id)
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)

    @test.create_mocks({api.sahara: ('client',
                                     'cluster_template_get',
                                     'cluster_template_update',
                                     'plugin_get_version_details',
                                     'nodegroup_template_find')})
    def test_update(self):
        ct = self.cluster_templates.first()
        ngts = self.nodegroup_templates.list()
        configs = self.plugins_configs.first()
        new_name = "UpdatedName"
        new_ct = copy.copy(ct)
        new_ct.name = new_name
        self.mock_cluster_template_get.return_value = ct
        self.mock_plugin_get_version_details.return_value = configs
        self.mock_nodegroup_template_find.return_value = ngts
        self.mock_cluster_template_update.return_value = new_ct

        url = reverse('horizon:project:data_processing.clusters:ct-edit',
                      args=[ct.id])

        def serialize(obj):
            return utils.serialize(jsonutils.dump_as_bytes(obj))

        res = self.client.post(
            url,
            {'ct_id': ct.id,
             'cluster_template_name': new_name,
             'plugin_name': ct.plugin_name,
             'hadoop_version': ct.hadoop_version,
             'description': ct.description,
             'hidden_configure_field': "",
             'template_id_0': ct.node_groups[0]['node_group_template_id'],
             'group_name_0': ct.node_groups[0]['name'],
             'count_0': 1,
             'serialized_0': serialize(ct.node_groups[0]),
             'template_id_1': ct.node_groups[1]['node_group_template_id'],
             'group_name_1': ct.node_groups[1]['name'],
             'count_1': 2,
             'serialized_1': serialize(ct.node_groups[1]),
             'forms_ids': "[0,1]",
             })

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)
        self.mock_cluster_template_update.assert_called_once_with(
            request=test.IsHttpRequest(),
            ct_id=ct.id,
            name=new_name,
            plugin_name=ct.plugin_name,
            hadoop_version=ct.hadoop_version,
            description=ct.description,
            cluster_configs=ct.cluster_configs,
            node_groups=ct.node_groups,
            anti_affinity=ct.anti_affinity,
            use_autoconfig=False,
            shares=ct.shares,
            is_public=False,
            is_protected=False,
            domain_name=ct.domain_name
        )



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

import sahara_dashboard.content.data_processing. \
    jobs.views as views
import sahara_dashboard.content.data_processing. \
    jobs.job_binaries.views as job_binary_views
import sahara_dashboard.content.data_processing. \
    jobs.data_sources.views as data_source_views
import sahara_dashboard.content.data_processing. \
    jobs.job_templates.views as job_templates_views
import sahara_dashboard.content.data_processing. \
    jobs.wizard.views as job_wizard_views
import sahara_dashboard.content.data_processing. \
    jobs.jobs.views as jobs_views


urlpatterns = [re_path(r'^$', views.IndexView.as_view(), name='index'),
               re_path(r'^$', views.IndexView.as_view(), name='jobs'),
               re_path(r'^\?tab=job_tabs__jobs_tab$',
                       views.IndexView.as_view(),
                       name='jobs-tab'),
               re_path(r'^create-job$',
                       job_templates_views.CreateJobView.as_view(),
                       name='create-job'),
               re_path(r'^launch-job$',
                       job_templates_views.LaunchJobView.as_view(),
                       name='launch-job'),
               re_path(r'^launch-job-new-cluster$', job_templates_views.
                       LaunchJobNewClusterView.as_view(),
                       name='launch-job-new-cluster'),
               re_path(r'^choose-plugin$',
                       job_templates_views.ChoosePluginView.as_view(),
                       name='choose-plugin'),
               re_path(r'^job-template/(?P<job_id>[^/]+)$',
                       job_templates_views.
                       JobTemplateDetailsView.as_view(), name='jt-details'),
               re_path(r'^job-execution/(?P<job_execution_id>[^/]+)$',
                       jobs_views.JobDetailsView.as_view(), name='details'),
               re_path(r'^create-job-binary$',
                       job_binary_views.CreateJobBinaryView.as_view(),
                       name='create-job-binary'),
               re_path(r'^job-binary/(?P<job_binary_id>[^/]+)$',
                       job_binary_views.JobBinaryDetailsView.as_view(),
                       name='jb-details'),
               re_path(r'^job-binary/(?P<job_binary_id>[^/]+)/edit$',
                       job_binary_views.EditJobBinaryView.as_view(),
                       name='edit-job-binary'),
               re_path(r'^job-binary/(?P<job_binary_id>[^/]+)/download/$',
                       job_binary_views.DownloadJobBinaryView.as_view(),
                       name='download'),
               re_path(r'^create-data-source$',
                       data_source_views.CreateDataSourceView.as_view(),
                       name='create-data-source'),
               re_path(r'^data-source/(?P<data_source_id>[^/]+)/edit$',
                       data_source_views.EditDataSourceView.as_view(),
                       name='edit-data-source'),
               re_path(r'^data-source/(?P<data_source_id>[^/]+)$',
                       data_source_views.DataSourceDetailsView.as_view(),
                       name='ds-details'),
               re_path(r'^jobex_guide$',
                       job_wizard_views.JobExecutionGuideView.as_view(),
                       name='jobex_guide'),
               re_path(r'^jobex_guide/(?P<reset_jobex_guide>[^/]+)/$',
                       job_wizard_views.ResetJobExGuideView.as_view(),
                       name='reset_jobex_guide'),
               re_path(r'^job_type_select$',
                       job_wizard_views.JobTypeSelectView.as_view(),
                       name='job_type_select'),
               ]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import functools

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import forms
from horizon import tables


MESSAGE_MAPPING_PRESENT = {
    'public': functools.partial(
        ngettext_lazy,
        u"Make public",
        u"Make public"),
    'private': functools.partial(
        ngettext_lazy,
        u"Make private",
        u"Make private"),
    'protected': functools.partial(
        ngettext_lazy,
        u"Make protected",
        u"Make protected"),
    'unprotected': functools.partial(
        ngettext_lazy,
        u"Make unprotected",
        u"Make unprotected"),
}

MESSAGE_MAPPING_PAST = {
    'public': functools.partial(
        ngettext_lazy,
        u"Made public",
        u"Made public"),
    'private': functools.partial(
        ngettext_lazy,
        u"Made private",
        u"Made private"),
    'protected': functools.partial(
        ngettext_lazy,
        u"Made protected",
        u"Made protected"),
    'unprotected': functools.partial(
        ngettext_lazy,
        u"Made unprotected",
        u"Made unprotected"),
}


class RuleChangeAction(tables.BatchAction):
    rule = "no-rule"

    def action_present(self, count):
        return MESSAGE_MAPPING_PRESENT[self.rule](count)

    def action_past(self, count):
        return MESSAGE_MAPPING_PAST[self.rule](count)

    def action(self, request, datum_id):
        try:
            update_kwargs = {}
            if self.rule in ['public', 'private']:
                update_kwargs['is_public'] = self.rule == "public"
            if self.rule in ["protected", "unprotected"]:
                update_kwargs['is_protected'] = self.rule == "protected"
            self.change_rule_method(request, datum_id, **update_kwargs)
        except Exception as e:
            exceptions.handle(request, e)
            raise


class MakePublic(RuleChangeAction):
    name = "make_public"
    rule = "public"

    @abc.abstractmethod
    def change_rule_method(self, request, datum_id, **update_kwargs):
        pass


class MakePrivate(RuleChangeAction):
    name = "make_private"
    rule = "private"

    @abc.abstractmethod
    def change_rule_method(self, request, datum_id, **update_kwargs):
        pass


class MakeProtected(RuleChangeAction):
    name = "make_protected"
    rule = "protected"

    @abc.abstractmethod
    def change_rule_method(self, request, datum_id, **update_kwargs):
        pass


class MakeUnProtected(RuleChangeAction):
    name = "make_unprotected"
    rule = "unprotected"

    @abc.abstractmethod
    def change_rule_method(self, request, datum_id, **update_kwargs):
        pass


def get_is_public_form(object_type):
    return forms.BooleanField(
        label=_("Public"),
        help_text=_("If selected, %s will be shared across the "
                    "tenants") % object_type,
        required=False,
        widget=forms.CheckboxInput(),
        initial=False,
    )


def get_is_protected_form(object_type):
    return forms.BooleanField(
        label=_("Protected"),
        help_text=_("If selected, %s will be protected from modifications "
                    "until this will be unselected") % object_type,
        required=False,
        widget=forms.CheckboxInput(),
        initial=False)
